#!/usr/bin/perl -w
# V2.1
use strict;
use warnings;

#Utilisation du package Date::Manip pour les calculs sur les dates
# ----------------------------------------------------------------
use Date::Manip qw(ParseDate DateCalc Date_Cmp UnixDate);

#Utilisation du package List pour les sommes d'elements de tableaux
# -----------------------------------------------------------------
use List::Util qw(sum);

#Utilisation du package DBI pour les requetes mySQL
# -------------------------------------------------
use DBI;


# Debug function
our $LOCACC_DEBUG_LEVEL=1000;
# Localisation of config file
our $CONFIGURATION_FILE="/etc/pbsmaui2mysql.in";


sub debug_print {
    my ($mystring,$mylevel)=@_;
    if ($mylevel<$LOCACC_DEBUG_LEVEL) {print "\n$mystring\n";}
}


sub ymd_date {
    my ($somedate)=@_;
    return UnixDate($somedate,"%Y-%m-%d");
}

#######################################
# TRANSFORMATION HH:MM:SS EN SECONDES #
#######################################
sub time_to_sec {
        my ($hours,$min,$secs)=split(/:/,$_[0]);
        return ($hours * 3600) + ($min * 60) + $secs;
}

#############################################
# SUPPRESSION DE KB POUR LES CHAMPS MEMOIRE #
#############################################
sub kb_to_num {
        (my $memory = $_[0]) =~ s/kb//;
        return $memory;
}

###############################################
# GESTION AUTOMATIQUE DE LA COULEUR DES ROLES #
###############################################
sub set_role_color {
         my @color = ("slateblue1",
                      "orange:1.1",
                      "green:1.4",
                      "orchid1",
                      "tomato1",
                      "rosybrown4:1.1",
                      "aqua",
                      "darkolivegreen1:1.1",
                      "royalblue:1.2",
                      "orangered3:1.5",
                      "indianred1:1.2");
        return @color;
}

# ---------------------------------------------------------------------------------
# Read config file
# ---------------------------------------------------------------------------------
sub read_config_file {
    my %p2mconfig = (
        dbserver => "",
        dbname   => "",
        dblogin   => "",
        dbpasswd => "",
        pathpbs  => "",
        pathmaui => ""
    );
        
    ####################################################################################
    # Lecture du fichier d'entree pour connexion base et localisation logs TORQUE/MAUI #
    ####################################################################################
    my $infic=$CONFIGURATION_FILE;
    #print "Fichier d'entree = $infic\n";

    open (FILEIN,"<$infic");
    map { chomp } my @fic = <FILEIN>;
    close FILEIN;

    #print "$fic[1]:$fic[3]:$fic[5]:$fic[7]:$fic[9]:$fic[11]\n";
    #print "\n";
    $p2mconfig{dbserver} = $fic[ 1];
    $p2mconfig{dbname  } = $fic[ 3];
    $p2mconfig{dblogin } = $fic[ 5];
    $p2mconfig{dbpasswd} = $fic[ 7];
    $p2mconfig{pathpbs } = $fic[ 9];
    $p2mconfig{pathmaui} = $fic[11];
    
    return \%p2mconfig;
}

# ---------------------------------------------------------------------------------
# Dump current content of config
# ---------------------------------------------------------------------------------
sub dump_config {
    my %p2mconfig = @_;
    
    while ( my ($key, $value) = each(%p2mconfig) ) {
        print "$key => $value\n";
    }
}

# ---------------------------------------------------------------------------------
# Check if metainfo table exists, create it with if not.
# If metatable doesnt exists, assume that current datablase version is 1.3
# latest version without metainfo table.
# Returns current version of teh database according to metainfo.
# ---------------------------------------------------------------------------------
sub check_or_create_metainfo_table {
    my ($dbh)=@_;
    
    #print "\nici: $dbh\n";
        
    # Recherche de la table metainfo
    # ------------------------------
    my $sth = $dbh->prepare("SHOW TABLES")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    #my $resfirst = $sth->fetchrow_array;
    my $res = "";
    while (my @ligne = $sth->fetchrow_array and $res ne "metainfo") {
        $res = $ligne[0];
        #  print "$ligne[0]\n";
    }
    my $rc = $sth->finish;

    if ($res ne "metainfo") {
        print "\nLa table metainfo n'existe pas, elle va etre creee et remplie automatiquement\n";
        $sth = $dbh->do("CREATE TABLE metainfo
                       (variable VARCHAR(20) NOT NULL ,
                        value    VARCHAR(30) NOT NULL)");
        $sth = $dbh->do("ALTER TABLE metainfo ADD PRIMARY KEY (variable)");
        $sth = $dbh->do("INSERT INTO metainfo (variable,value)
                        VALUES ('Version','1.3'),('Date','2007-10-16')");
    }
    
    
    # Recherche de la version de la base
    # ----------------------------------
    $sth = $dbh->prepare("SELECT * FROM metainfo")
          or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
          or die "Couldn't execute statement: " . $sth->errstr;
    my $version = "";
    while (my @resultat = $sth->fetchrow_array) {
        if ($resultat[0] eq "Version") {
            $version = $resultat[1];
        }
    }
    $rc = $sth->finish;
    print "Version de la base de donnees accounting : $version\n";

    return $version;
}

# ---------------------------------------------------------------------------------
# Version 1.3 to 1.4 update
# -------------------------------
sub db_update_13_to14 {
    my($dbh)=@_;
    
    my $version="1.4";
    
    print "Passage automatique de la version 1.3 a la version 1.4\n";
    
    # Suppression champ exec_host dans table pbs car inutile
    $dbh->do("ALTER TABLE pbs DROP exec_host ");

    # Suppression champ hosts dans table maui car inutile
    $dbh->do("ALTER TABLE maui DROP hosts ");

    # Unicite sur user+groupe dans table users car certains users locaux ont plusieurs groupes
    $dbh->do("ALTER TABLE users DROP INDEX user , ADD UNIQUE user (user,groupe) ");

    # Ajout champ groupe dans table roles
    $dbh->do("ALTER TABLE roles ADD groupe VARCHAR( 16 ) NOT NULL AFTER role  ");

    # Unicite sur role+groupe dans table roles car certains roles locaux ont plusieurs groupes
    $dbh->do("ALTER TABLE roles DROP INDEX role , ADD UNIQUE role (role,groupe) ");

    # Mise a jour champ groupe dans table roles a partir des valeurs de la table users
    $dbh->do("UPDATE roles JOIN users ON users.role = roles.role SET roles.groupe = users.groupe ");

    # Ajout champ couleur dans table users
    $dbh->do("ALTER TABLE users ADD couleur VARCHAR( 20 ) NOT NULL COMMENT 'manuel' ");

    # Mise a jour champ couleur dans table users a partir des valeurs de la table roles
    $dbh->do("UPDATE users JOIN roles ON roles.role = users.role SET users.couleur = roles.couleur ");

    # Unicite sur yearmonth+user+groupe dans table pbsmois car certains users locaux ont plusieurs groupes
    $dbh->do("ALTER TABLE pbsmois DROP INDEX yearmonth , ADD UNIQUE yearmonth (yearmonth,useri,groupe) "); 
    
    $dbh->do("update metainfo set value='1.4' where variable='Version' ");
    $dbh->do("update metainfo set value='2007-11-14' where variable='Date' ");
    $version = "1.4";
    print "Nouvelle version de la base de donnees accounting : $version\n";

}

# ---------------------------------------------------------------------------------
# Version 1.4 to 1.6 update
# -------------------------------
sub db_update_14_to16 {
    my($dbh)=@_;

    my $version="1.6";

    print "Passage automatique de la version 1.4 a la version 1.6\n";

    # Ajout champ first_occur dans table groupes pour optimisation requetes
    $dbh->do("ALTER TABLE groupes ADD first_occur DATE NOT NULL AFTER queue ");

    # Modif valeur par defaut de localqueue
    $dbh->do("ALTER TABLE groupes CHANGE localqueue localqueue ENUM( 'no', 'yes', '' )
              CHARACTER SET utf8 COLLATE utf8_unicode_ci NOT NULL DEFAULT 'no' COMMENT 'manuel' ");

    $dbh->do("update metainfo set value='1.6' where variable='Version' ");
    $dbh->do("update metainfo set value='2008-03-19' where variable='Date' ");
    $version = "1.6";
    print "Nouvelle version de la base de donnees accounting : $version\n";

}

# ---------------------------------------------------------------------------------
# Version 1.6 to 2.0 update
# -------------------------------
sub db_update_16_to20 {
    my($dbh)=@_;
    
    my $version="2.0";
    
    print "Passage automatique de la version 1.6 a la version 2.0\n";

    # Passage de 16 a 32 caracteres maxi pour champ classe table classes et table maui
    $dbh->do("ALTER TABLE `classes` CHANGE `classe` `classe` VARCHAR( 32 ) CHARACTER SET utf8 COLLATE utf8_unicode_ci NOT NULL ");
    $dbh->do("ALTER TABLE `maui` CHANGE `classe` `classe` VARCHAR( 32 ) CHARACTER SET utf8 COLLATE utf8_unicode_ci NOT NULL DEFAULT 'none' COMMENT 'Rang 8' "); 

    # Correction du nom de la classe pour VO CTA dans table maui
    $dbh->do("UPDATE `maui` SET `classe` = '[vo.cta.in2p3.fr:1]' WHERE `classe` LIKE '[vo.cta.in2p3.fr' ");
 
    # Changement de nom pour la queue du groupe rag_gr
    $dbh->do("UPDATE `groupes` SET `queue` = 'rhal' WHERE CONVERT( `groupes`.`groupe` USING utf8 ) = 'rag_gr' LIMIT 1 ");

    # Reordonnement table mpi
    $dbh->do("ALTER TABLE `mpi` ORDER BY `nbprocs` "); 

    # Incident table calcul
    $dbh->do("UPDATE `calcul` SET `incident` = 'Firmware update' WHERE `calcul`.`date` = '2009-01-15' AND CONVERT( `calcul`.`servername` USING utf8 ) = 'lapp-ce01.in2p3.fr' LIMIT 1 ");

    # Longueur du champ account dans la table maui
    $dbh->do("ALTER TABLE `maui` CHANGE `account` `account` VARCHAR( 12 ) CHARACTER SET utf8 COLLATE utf8_unicode_ci NOT NULL DEFAULT 'none' COMMENT 'Rang 26' ");

    # Ajout des champs first_occur et closed dans la table users
    $dbh->do("ALTER TABLE `users` ADD `first_occur` DATE NOT NULL ");
    $dbh->do("ALTER TABLE `users` ADD `closed` DATE NOT NULL ");

    # Suppression du champ couleur dans la table users
    $dbh->do("ALTER TABLE `users` DROP `couleur`  ");

    # Ajout du champ legende dans la table roles
    $dbh->do("ALTER TABLE `roles` ADD `legende` VARCHAR( 48 ) NOT NULL COMMENT 'manuel' ");

    $dbh->do("update metainfo set value='2.0' where variable='Version' ");
    $dbh->do("update metainfo set value='2008-12-02' where variable='Date' ");
    $version = "2.0";
    print "Nouvelle version de la base de donnees accounting : $version\n";

}

# ---------------------------------------------------------------------------------
# If database is not up to date (latest version)
# process for the updates.
# ---------------------------------------------------------------------------------
sub update_database_if_needed {
    my ($dbh,$dbversion)=@_;
    
    # Current version: nothing to do
    if ($dbversion == "2.0") {return;}

    if ($dbversion == "1.6") {
        db_update_16_to20($dbh);
    }

    if ($dbversion == "1.4") {
        db_update_14_to16($dbh);
    }
    
    if ($dbversion == "1.3") {
        db_update_13_to14($dbh);
    }
}

# ---------------------------------------------------------------------------------
# Open the database
# Then test for its version, update if needed.
# ---------------------------------------------------------------------------------
sub open_database {
    my ($p2mconfig) = @_;
    
    #######################
    # Connection a la BDD #
    #######################
    print "Connection a la BDD\n";
    print "-------------------\n";
    print "\n";
    
    #print "DBI->connect(
    #    DBI:mysql:$p2mconfig{dbname}:$p2mconfig{dbserver},
    #    $p2mconfig{dblogin},
    #    $p2mconfig{dbpasswd});";

    my $dbh = DBI->connect(
        "DBI:mysql:$p2mconfig->{dbname}:$p2mconfig->{dbserver}",
        "$p2mconfig->{dblogin}",
        "$p2mconfig->{dbpasswd}");
    
    #print "\nla: $dbh \n";
    
    my $dbversion=check_or_create_metainfo_table($dbh); 
    update_database_if_needed($dbh,$dbversion);
    
    return $dbh;
}

# ---------------------------------------------------------------------------------
# Returns number of nodes and processors (physical and virtuals)
# ---------------------------------------------------------------------------------
sub get_nb_nodes_procs_vprocs {
    my @liste_procs = `pbsnodes -a | grep -w ncpus`;
    my $nodes = 1 + $#liste_procs;
    
    my $procs = "0";
    foreach my $line (@liste_procs) {
        my @text = split(/,/,$line);
        foreach my $el (@text) {
            (my $np=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^ncpus=/;
            $procs += $np if $el =~ /^ncpus=/;
        }
    }
    
    # Calcul du nombre de processeurs virtuels
    # ----------------------------------------
    my @liste_vprocs = `pbsnodes -a | grep -w np`;
    my $vprocs = 0;
    foreach my $vline (@liste_vprocs) {
    $vline =~ s/^\s+//;
    my @vnp = split(/ /,$vline);
    $vprocs += $vnp[2];
    }
    
    return ($nodes,$procs,$vprocs);
}


sub get_first_entry_date {
    my ($p2mconfig,$dbh) = @_;
    # Determination du premier jour d'enregistrement dans la base
    # -----------------------------------------------------------
    my $firstmon = "";
    my $sth = $dbh->prepare("select min(date) from calcul")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $resfirst = $sth->fetchrow_array;
    if ($resfirst ne "") {
        $firstmon = &ParseDate($resfirst);
    } else {
        my $firstfic = `ls -1r $p2mconfig->{pathpbs} |tail -1`;
        if ($firstfic eq "") {
            print "Aucun fichier de log de TORQUE dans le repertoire $p2mconfig->{pathpbs}\n";
            print "=> Pas de mise a jour de la base de donnees $p2mconfig->{dbname}\n";
            exit;
        } else {
            print "Premier fichier de log de TORQUE trouve : $firstfic";
            $firstmon = &DateCalc($firstfic,"- 1days");
        }
    }
    my $rc = $sth->finish;
    return ($firstmon);
}


sub get_last_entry_date {
    my ($dbh,$firstmon) = @_;
    # Determination du dernier jour d'enregistrement dans la base
    # -----------------------------------------------------------
    my $lastmon = "";
    my $sth = $dbh->prepare("select max(date) from calcul")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $reslast = $sth->fetchrow_array;
    if ($reslast ne "") {
        $lastmon = &ParseDate($reslast);
    } else {
        $lastmon = $firstmon;
    }
    my $rc = $sth->finish;
    return ($lastmon);
}


sub update_calcul_table {
    my ($dbh,$nextmon,$veille,$servername)=@_;
    #########################################################################################
    # Mise a jour de la table calcul avec dates manquantes si serveur OFF au moment du CRON #
    ######################################################################################### 

    # Calcul du nombre de noeuds et de processeurs
    # ------------------------------------------------------
    my ($nodes,$procs,$vprocs)=get_nb_nodes_procs_vprocs();
    print "Nombre de noeuds de calcul      = $nodes \n";
    print "Nombre de processeurs physiques = $procs \n";
    print "Nombre de processeurs virtuels  = $vprocs\n";
    print "\n";
        
    # Mise a jour de la table calcul pour toutes les dates 
    # entre le dernier jour d'enregistrement et la veille
    # ----------------------------------------------------
    print "\n";
    print "Mise a jour de la table calcul\n";
    print "------------------------------\n";
    for ( my $newmon = $nextmon ; 
          Date_Cmp($newmon,$veille) <= 0 ;
          $newmon = DateCalc($newmon,"+ 1days") ) {
        my $newmonq = UnixDate($newmon,"%Y-%m-%d");
        print "Nouvelle date d'enregistrement = $newmonq\n";
        $dbh->do("insert into calcul (date,nodes,procs,vprocs,servername)
                  values ('$newmonq','$nodes','$procs','$vprocs','$servername') ");
    }

    # Nombre de dates dans la table apres mise a jour
    # -----------------------------------------------
    my $sth = $dbh->prepare("select * from calcul")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $rc = $sth->finish;
    my $nbcalcul_new = $sth->rows;

    return ($nbcalcul_new);
}


sub update_pbs_table {
    my ($p2mconfig,$dbh,$nextmon,$veille) =@_;

    ######################################################################################
    # Mise a jour de la table PBS pour toutes les dates depuis le dernier enregistrement #
    ######################################################################################
    
    # Nombre d'enregistrements dans la table avant mise a jour
    # --------------------------------------------------------
    my $sth = $dbh->prepare("select * from pbs")
    or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
    or die "Couldn't execute statement: " . $sth->errstr;
    my $rc = $sth->finish;
    my $nbpbs_old = $sth->rows;
    
    # Ajout des nouveaux enregistrements
    # ----------------------------------
    print "\n";
    print "Mise a jour de la table pbs\n";
    print "---------------------------\n";
    for ( my $newmon = $nextmon ;
          &Date_Cmp($newmon,$veille) <= 0 ;
          $newmon = &DateCalc($newmon,"+ 1days") ) {
        my $pbsmon  = &UnixDate($newmon,"%Y%m%d");
        
        # Fichier de log de PBS a exploiter
        # ---------------------------------
        my $inputfile = "$p2mconfig->{pathpbs}/$pbsmon";
        #  print "Logfile TORQUE = $inputfile\n";
        print "\n";
        
        open (FILEIN,"<$inputfile");
        map { chomp } my @lines = <FILEIN>;
        close FILEIN;
        
        foreach my $line (@lines) {
            my @parts = split(/;/,$line);
            if ( $parts[1] eq "E" ) {
            my ($date,$time)=split(/ /,$parts[0]);
            map { chomp } my $datenum = `/bin/date -d $date +%Y-%m-%d`;
            my $times = time_to_sec($time);
        #      ($month,$day,$year)=split(/\//,$date);
        #      ($jobid)=split(/\./,$parts[2]);
            my ($jobid,$server,$domain,$country)=split(/\./,$parts[2]);
            my $serverfullname = "$server.$domain.$country";
            my @text = split(/ /,$parts[3]);
            my $user = "";
            my $groupe = "";
            my $jobname = "";
            my $queue = "";
            my $ctime = "0";
            my $qtime = "0";
            my $etime = "0";
            my $starttime = "";
            my $exec_host = "";
            my $nbprocs = "";
            my $endtime = "";
            my $exit_status = "";
            my $cputs = "0";
            my $mem = "0";
            my $vmem = "0";
            my $walltimes = "0";
            foreach my $el (@text) {
                ($user=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^user=/;
                ($groupe=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^group=/;
                ($jobname=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^jobname=/;
                ($queue=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^queue=/;
                ($ctime=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^ctime=/;
                ($qtime=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^qtime=/;
                ($etime=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^etime=/;
                ($starttime=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^start=/;
                ($exec_host=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^exec_host=/;
                my @split_host = split(/\+/,$exec_host) if $el =~ /^exec_host=/;
                $nbprocs = 1 + $#split_host if $el =~ /^exec_host=/;
        #        ($nodect=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^Resource_List.nodect=/;
                ($endtime=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^end=/;
                ($exit_status=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^Exit_status=/;
                (my $cput=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^resources_used.cput=/;
                $cputs = time_to_sec($cput) if $el =~ /^resources_used.cput=/;
                (my $memkb=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^resources_used.mem=/;
                $mem = kb_to_num($memkb) if $el =~ /^resources_used.mem=/;
                (my $vmemkb=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^resources_used.vmem=/;
                $vmem = kb_to_num($vmemkb) if $el =~ /^resources_used.vmem=/;
                (my $walltime=$el)=~s/(^.+=)(.*)/$2/ if $el =~ /^resources_used.walltime=/;
                $walltimes = time_to_sec($walltime) if $el =~ /^resources_used.walltime=/;
            }
        
        #     Affichage des jobs passant en batch et des jobs paralleles passant par la grille
        #     --------------------------------------------------------------------------------
        #    if ($queue eq 'flash' || $queue eq 'local' ||  $nbprocs > 1) {
        #        print "L'utilisateur $user a tourne le job $jobid sur $nbprocs procs dans la queue $queue\n";
        #        print "\n";
        #    }
        
        #     Affichage des jobs termines en erreur
        #     -------------------------------------
        #    if ($exit_status != 0) {
        #        print "ATTENTION !!!!!\n";
        #        print "Erreur $exit_status pour le job $jobid soumis par $user\n";
        #        print "\n";
        #    }
        
        #     Verification coherence groupe et queue
        #     --------------------------------------
            if ($groupe ne $queue && $queue ne 'local' && $queue ne 'flash' &&
                !($groupe eq 'atlas_gr'  && $queue eq 'atlas')  &&
                !($groupe eq 'lhcb_gr'   && $queue eq 'lhcb')   &&
                !($groupe eq 'geant4_gr' && $queue eq 'geant4') &&
                !($groupe eq 'forma_gr'  && $queue eq 'formation') &&
                !($groupe eq 'forma_gr'  && $queue eq 'vo.formation.id') &&
                !($groupe eq 'lapp_gr'   && $queue eq 'lapp')   &&
                !($groupe eq 'lapp_gr'   && $queue eq 'vo.lapp.in2p3.f')   &&
                !($groupe eq 'rag_gr'    && $queue eq 'rag')    &&
                !($groupe eq 'rag_gr'    && $queue eq 'rhal')   &&
                !($groupe eq 'rag_gr'    && $queue eq 'vo.rhonealp')    &&
                !($groupe eq 'rag_gr'    && $queue eq 'vo.rhone_alpes')    &&
                !($groupe eq 'cta_gr'    && $queue eq 'cta')    &&
                !($groupe eq 'cta_gr'    && $queue eq 'vo.cta.in2p3.fr') ) {
                print "ATTENTION !!!!!\n";
                print "Nom de la queue incoherent avec le groupe\n";
                print "jobid = $jobid user = $user groupe = $groupe et queue = $queue\n";
                print "\n";
            }
        
        #     Verification coherence dates
        #     ----------------------------
            my $date_endtime = &ParseDate("epoch $endtime");
            my $heure_endtime = &UnixDate($date_endtime,"%T");
            $date_endtime = &UnixDate($date_endtime,"%Y-%m-%d");
            if ($date_endtime ne $datenum) {
                print "ATTENTION !!!!!\n";
                print "Incoherence sur les dates dans la table pbs\n";
                print "jobid = $jobid date = $datenum et date_endtime = $date_endtime\n";
                print "heure_endtime = $heure_endtime et walltimes = $walltimes\n";
                print "\n";
            }
        
        #     Verification pour valeurs de ctime et qtime
        #     -------------------------------------------
            if (($qtime-$ctime) > 5) {
                print "ATTENTION !!!!!\n";
                print "jobid = $jobid ctime = $ctime qtime = $qtime\n";
                print "Ecart = ".($qtime-$ctime)."\n";
                print "\n";
            }
        
        #     Verification pour valeurs de qtime et etime
        #     -------------------------------------------
            if (($etime-$qtime) != 0) {
                print "ATTENTION !!!!!\n";
                print "jobid = $jobid qtime = $qtime etime = $etime\n";
                print "Ecart = ".($etime-$qtime)."\n";
                print "\n";
            }
        
        #     Verification pour valeurs de resources_used.walltime
        #     ----------------------------------------------------
            if ($walltimes < 0) {
                print "ATTENTION !!!!!\n";
                print "jobid = $jobid walltimes = $walltimes est negatif\n";
                print "starttime = $starttime et endtime = $endtime\n";
                print "Calcul a partir de (endtime-starttime) = ".($endtime-$starttime)."\n";
                print "\n";
                $walltimes = $endtime - $starttime;
            }
            elsif (abs($endtime - $starttime - $walltimes) > 5) {
                print "ATTENTION !!!!!\n";
                print "jobid = $jobid starttime = $starttime et endtime = $endtime\n";
                print "walltimes = $walltimes different de (endtime-starttime) = ".($endtime-$starttime)."\n";
                print "Ecart = ".($endtime-$starttime-$walltimes)."\n";
                print "\n";
            }
        
        #     Decomposition de walltimes par jour
        #     -----------------------------------
            my $wt_temp = $walltimes;
            my $t_temp  = $times;
            my $walltimesplit = "";
            while ($wt_temp >= $t_temp) {
                $walltimesplit .= $t_temp."+";
                $wt_temp -= $t_temp;
                $t_temp = 86400;
            }
            $walltimesplit .= $wt_temp;
        
        
        #      print "$jobid;$datenum;$time;$user;$groupe;$jobname;$queue;$exit_status;\n";
        #      print "$serverfullname;$exec_host;$nbprocs;$mem;$vmem;\n";
        #      print "$ctime;$qtime;$etime;$starttime;$endtime;$walltimes;\n";
        #      print "$walltimesplit;$cputs;\n";
        
            $dbh->do("insert into pbs (jobid,date,heure,user,groupe,jobname,queue,exit_status,servername,nbprocs,
                    mem,vmem,ctime,qtime,etime,starttime,endtime,walltime,walltimesplit,cputime) values
                    ('$jobid','$datenum','$time','$user','$groupe','$jobname','$queue','$exit_status',
                    '$serverfullname','$nbprocs','$mem','$vmem','$ctime','$qtime','$etime',
                    '$starttime','$endtime','$walltimes','$walltimesplit','$cputs') ");

            $dbh->do("insert into pbsmaui (jobid,date,heure,user,groupe,jobname,queue,exit_status,servername,nbprocs,
                    mem,vmem,ctime,qtime,etime,starttime,endtime,walltime,walltimesplit,cputime) values
                    ('$jobid','$datenum','$time','$user','$groupe','$jobname','$queue','$exit_status',
                    '$serverfullname','$nbprocs','$mem','$vmem','$ctime','$qtime','$etime',
                    '$starttime','$endtime','$walltimes','$walltimesplit','$cputs') ");
            }
        }

    #   Affichage des jobs passant en batch et des jobs MPI
    #   ---------------------------------------------------
        my $newmon_ymd = &UnixDate($newmon,"%Y-%m-%d");
        my $newmon_dmy = &UnixDate($newmon,"%d-%m-%Y");
        $sth = $dbh->prepare("select user,groupe,count(jobid),nbprocs,queue from pbs where date like '$newmon_ymd'
                              group by user,groupe,nbprocs,queue")
        or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
        my $tmpuser    = "";
        my $tmpgroupe  = "";
        my $tmpnbjobs  = 0;
        my $tmpnbprocs = 0;
        my $tmpqueue   = "";
        while (my @result = $sth->fetchrow_array) {
            $tmpuser    = $result[0];
            $tmpgroupe  = $result[1];
            $tmpnbjobs  = $result[2];
            $tmpnbprocs = $result[3];
            $tmpqueue   = $result[4];
            if ($tmpqueue eq 'flash' || $tmpqueue eq 'local' || $tmpnbprocs > 1) {
                print "Le $newmon_dmy, l'utilisateur $tmpuser du groupe $tmpgroupe a tourne $tmpnbjobs jobs sur $tmpnbprocs procs dans la queue $tmpqueue\n";
            }
        }
        $rc = $sth->finish;    
        print "\n";

    #   Affichage des jobs passant en erreur
    #   ------------------------------------
        $sth = $dbh->prepare("select user,groupe,count(jobid),exit_status from pbs where date like '$newmon_ymd'
                              group by user,groupe,exit_status")
        or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
        my $tmp2user   = "";
        my $tmp2groupe = "";
        my $tmp2nbjobs = 0;
        my $tmp2exit   = 0;
        my $tmp2nberr  = 0;
        my $tmp2nbok   = 0;
        my $tmp2nbtot  = 0;
        my $tmp2perc   = 0;
        while (my @result = $sth->fetchrow_array) {
            if ($result[0] ne $tmp2user || $result[1] ne $tmp2groupe) {
              if ($tmp2user ne "" && $tmp2groupe ne "") {
                $tmp2nbtot  = $tmp2nbok + $tmp2nberr;
                $tmp2perc   = 100 * $tmp2nbok / $tmp2nbtot;
                printf "Le %s, l'utilisateur %s du groupe %s a reussi %6.2f %s de jobs sur %d jobs\n"
                    ,$newmon_dmy,$tmp2user,$tmp2groupe,$tmp2perc,"%",$tmp2nbtot;
              }
              $tmp2nberr  = 0;
              $tmp2nbok   = 0;
              $tmp2user   = $result[0];
              $tmp2groupe = $result[1];
            }
            $tmp2nbjobs = $result[2];
            $tmp2exit   = $result[3];
            if ($tmp2exit == 0) {
                $tmp2nbok  += $tmp2nbjobs;
            } else {
                $tmp2nberr += $tmp2nbjobs;
            }
        }
        if ($tmp2user ne "" && $tmp2groupe ne "") {
            $tmp2nbtot  = $tmp2nbok + $tmp2nberr;
            $tmp2perc   = 100 * $tmp2nbok / $tmp2nbtot;
            printf "Le %s, l'utilisateur %s du groupe %s a reussi %6.2f %s de jobs sur %d jobs\n"
                    ,$newmon_dmy,$tmp2user,$tmp2groupe,$tmp2perc,"%",$tmp2nbtot;
        }
        $rc = $sth->finish;
        print "\n";
    }
    
    # Nombre d'enregistrements dans la table apres mise a jour
    # --------------------------------------------------------
    $sth = $dbh->prepare("select * from pbs")
    or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
    or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbpbs_new = $sth->rows;
    
    # Nombre de nouveaux enregistrements
    # ----------------------------------
    my $nbpbs_delta = $nbpbs_new - $nbpbs_old;
    print "$nbpbs_delta enregistrements ont ete ajoutes\n";
    print "\n";

    return ($nbpbs_delta);
}


#sub update_pbs_group_table {
sub update_groupes_table {
    my ($dbh,$nextmon,$veille) =@_;
    
    ##############################################
    # Mise a jour de la table des groupes de PBS #
    ##############################################
    
    # Nombre de groupes dans la table groupes avant mise a jour
    # ---------------------------------------------------------
    my $sth = $dbh->prepare("select * from groupes")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $rc = $sth->finish;
    my $nbgroupe_old = $sth->rows;
    
    # Ajout des nouveaux groupes avec leur queue
    # ------------------------------------------
    print "\n";
    print "Mise a jour de la table groupes\n";
    print "-------------------------------\n";
    $sth = $dbh->prepare("select groupe,queue from pbs where date between '"
                        .ymd_date($nextmon)
                        ."' and '".ymd_date($veille)."' group by groupe")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    while (my @liste_groupe = $sth->fetchrow_array) {
        #  print "$liste_groupe[0] $liste_groupe[1]\n";
        $dbh->do("insert into groupes (groupe,queue) values 
                     ('$liste_groupe[0]','$liste_groupe[1]')
                  on duplicate key update groupe='$liste_groupe[0]'");
    }
    $rc = $sth->finish;
    
    # Nombre de groupes dans la table groupes apres mise a jour
    # ---------------------------------------------------------
    $sth = $dbh->prepare("select * from groupes")
    or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
    or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbgroupe_new = $sth->rows;
    
    # Affichage du nombre de nouveaux groupes
    # ---------------------------------------
    my $nbgroupe_delta = $nbgroupe_new - $nbgroupe_old;
    print "$nbgroupe_delta groupes ont ete ajoutes\n";
    
    # Affichage du nom des nouveaux groupes avec nom de la queue
    # ----------------------------------------------------------
    if ($nbgroupe_delta != 0) {
        my $nbgroupe_limit = $nbgroupe_new-1;
        my $first_occur=ymd_date($veille);
        $sth = $dbh->prepare("select * from groupes order by numero asc
                              limit $nbgroupe_old,$nbgroupe_limit")
            or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        # Mise a jour du champ first_occur pour les nouveaux groupes
        # ----------------------------------------------------------
        while (my @liste_new_groupe = $sth->fetchrow_array) {
            if ($liste_new_groupe[2] eq "0000-00-00") {
              $dbh->do(
                  "update groupes set first_occur='$first_occur' where groupe='$liste_new_groupe[0]' ");
            }
            print "$liste_new_groupe[0] $liste_new_groupe[1] $first_occur\n";
        }
        $rc = $sth->finish;
        
        # Affichage d'un warning pour mettre a jour 
        # surcharge, localqueue, numero et couleur
        # -----------------------------------------
        print "\n";
        print "NE PAS OUBLIER DE MODIFIER A LA MAIN LES CHAMPS surcharge, ";
        print "localqueue, numero, couleur et legende POUR CHAQUE NOUVEAU GROUPE\n";
    }

    return ($nbgroupe_new);
}


#sub update_users_table {
sub update_user_and_role_table {
    my ($dbh,$nextmon,$veille) = @_;
    
    #####################################################################
    # Mise a jour de la table des users et de la table des roles de PBS #
    #####################################################################
    
    # Nombre d'utilisateurs dans la table users avant mise a jour
    # -----------------------------------------------------------
    my $sth = $dbh->prepare("select * from users")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $rc = $sth->finish;
    my $nbuser_old = $sth->rows;
    
    # Nombre de roles dans la table roles avant mise a jour
    # -----------------------------------------------------
    $sth = $dbh->prepare("select * from roles")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbrole_old = $sth->rows;
    
    # Ajout des nouveaux utilisateurs avec leur groupe et leur role dans
    # la table users et ajout des nouveaux roles avec leur groupe dans la
    # table roles
    # -------------------------------------------------------------------
    $sth = $dbh->prepare("select user,pbs.groupe,localqueue from pbs join
                          groupes on pbs.groupe=groupes.groupe where date
                          between '".ymd_date($nextmon)
                          ."' and '".ymd_date($veille)."' group by user,pbs.groupe")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $role = "";
    my $couleur = "";
    my $legende = "";
    while (my @liste_user = $sth->fetchrow_array) {
        if ($liste_user[2] eq "yes") {
            $role = $liste_user[0];
            $couleur = "white";
        } else {
            ($role = $liste_user[0]) =~ s/\d+/x/;
            if ($role =~ /.*x$/) {
              $couleur = "green:1.4";
              $legende = "Role generique";
            } elsif ($role =~ /.*pi$/) {
              $couleur = "orchid1";
              $legende = "Role pilote";
            } elsif ($role =~ /.*p$/) {
              $couleur = "slateblue1";
              $legende = "Role production";
            } elsif ($role =~ /.*s$/) {
              $couleur = "orange:1.1";
              $legende = "Role software";
            }
        }
        #  print "$liste_user[0] $liste_user[1] $role\n";
        $dbh->do("insert into users (user,groupe,role) values
                    ('$liste_user[0]','$liste_user[1]','$role')
                  on duplicate key update user='$liste_user[0]'");
        $dbh->do("insert into roles (role,groupe,couleur,legende) values
                    ('$role','$liste_user[1]','$couleur','$legende')
                  on duplicate key update role='$role'");
    }
    $rc = $sth->finish;
    
    # Nombre d'utilisateurs dans la table users apres mise a jour
    # -----------------------------------------------------------
    $sth = $dbh->prepare("select * from users")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbuser_new = $sth->rows;
    
    # Affichage du nombre de nouveaux utilisateurs dans la table users
    # ----------------------------------------------------------------
    print "\n";
    print "Mise a jour de la table users\n";
    print "-----------------------------\n";
    my $nbuser_delta = $nbuser_new - $nbuser_old;
    print "$nbuser_delta utilisateurs ont ete ajoutes\n";
    
    # Affichage du nom des nouveaux utilisateurs avec
    # leurs groupe et role dans la table users
    # -------------------------------------------------
    if ($nbuser_delta != 0) {
        my $nbuser_limit = $nbuser_new-1;
        my $first_occur=ymd_date($veille);
        $sth = $dbh->prepare("select * from users limit $nbuser_old,$nbuser_limit")
            or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        # Mise a jour du champ first_occur pour les nouveaux utilisateurs
        # ----------------------------------------------------------
        while (my @liste_new_user = $sth->fetchrow_array) {
            if ($liste_new_user[3] eq "0000-00-00") {
              $dbh->do(
                  "update users set first_occur='$first_occur' where user='$liste_new_user[0]' ");
            }
            print "$liste_new_user[0] $liste_new_user[1] $liste_new_user[2] $first_occur\n";
        }
        $rc = $sth->finish;
    }
    
    # Nombre de roles dans la table roles apres mise a jour
    # -----------------------------------------------------
    $sth = $dbh->prepare("select * from roles")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbrole_new = $sth->rows;
    
    # Affichage du nombre de nouveaux roles dans la table roles
    # ---------------------------------------------------------
    print "\n";
    print "Mise a jour de la table roles\n";
    print "-----------------------------\n";
    my $nbrole_delta = $nbrole_new - $nbrole_old;
    print "$nbrole_delta roles ont ete ajoutes\n";
    
    # Affichage du nom des nouveaux roles dans la table roles
    # -------------------------------------------------------
    if ($nbrole_delta != 0) {
        my $nbrole_limit = $nbrole_new-1;
        $sth = $dbh->prepare("select * from roles limit $nbrole_old,$nbrole_limit")
            or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        while (my @liste_new_role = $sth->fetchrow_array) {
            if ($liste_new_role[2] eq "white") {
              my $sth2 = $dbh->prepare("select * from roles
                                        where groupe like '$liste_new_role[1]'
                                        order by role ASC")
                      or die "Couldn't prepare statement: " . $dbh->errstr;
              my $rv2 = $sth2->execute
                      or die "Couldn't execute statement: " . $sth2->errstr;
              my $ind = 0;
              my @tab_color = set_role_color();
              while (my @result = $sth2->fetchrow_array) {
                  $dbh->do(
                    "update roles set couleur='$tab_color[$ind]' where role='$result[0]'
                                                                 and groupe='$result[1]' ");
                  $ind += 1;
              }
              my $rc2 = $sth2->finish;
            }
        }
        $rc = $sth->finish;
        
        # Affichage d'un warning pour mettre a jour la couleur
        # ----------------------------------------------------
        my $sth3 = $dbh->prepare("select * from roles limit $nbrole_old,$nbrole_limit")
            or die "Couldn't prepare statement: " . $dbh->errstr;
        my $rv3 = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        while (my @liste_new_role_colored = $sth->fetchrow_array) {
            print "$liste_new_role_colored[0] $liste_new_role_colored[1] $liste_new_role_colored[2]\n";
            print "\nNE PAS OUBLIER DE CONTROLER ET MODIFIER A LA MAIN SI NECESSAIRE LE CHAMP legende POUR ";
            print "LE ROLE $liste_new_role_colored[0] DU GROUPE $liste_new_role_colored[1]\n";
            if ($liste_new_role_colored[2] eq "white") {
              print "\nNE PAS OUBLIER DE MODIFIER A LA MAIN LE CHAMP couleur POUR ";
              print "LE ROLE $liste_new_role_colored[0] DU GROUPE $liste_new_role_colored[1]\n";
            }
        }
        my $rc3 = $sth3->finish;
    }

    return ($nbrole_new,$nbuser_new);
    #return ($nbuser_new);
}


sub update_mpi_table {
    my ($dbh,$nextmon,$veille) = @_;
    
    ###############################
    # Mise a jour de la table MPI #
    ###############################
    
    # Nombre d'entrees dans la table mpi avant mise a jour
    # ----------------------------------------------------
    my $sth = $dbh->prepare("select * from mpi")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $rc = $sth->finish;
    my $nbmpi_old = $sth->rows;
    
    # Ajout des nouvelles entrees
    # ---------------------------
    $sth = $dbh->prepare("select nbprocs from pbs where date between '"
                         .ymd_date($nextmon)."' and '".ymd_date($veille)
                         ."' group by nbprocs")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    while (my @liste_nbproc = $sth->fetchrow_array) {
        #  print "$liste_nbproc[0]\n";
        $dbh->do("insert into mpi (nbprocs) values ('$liste_nbproc[0]')
                  on duplicate key update nbprocs='$liste_nbproc[0]'");
    }
    $rc = $sth->finish;
    
    # Nombre d'entrees dans la table mpi apres mise a jour
    # ----------------------------------------------------
    $sth = $dbh->prepare("select * from mpi")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbmpi_new = $sth->rows;
    
    # Affichage du nombre de nouvelles entrees
    # ----------------------------------------
    print "\n";
    print "Mise a jour de la table mpi\n";
    print "---------------------------\n";
    my $nbmpi_delta = $nbmpi_new - $nbmpi_old;
    print "$nbmpi_delta entrees ont ete ajoutees\n";
    
    # Affichage des valeurs des nouvelles entrees dans la table mpi
    # -------------------------------------------------------------
    if ($nbmpi_delta != 0) {
        my $nbmpi_limit = $nbmpi_new-1;
        $sth = $dbh->prepare("select * from mpi limit $nbmpi_old,$nbmpi_limit")
            or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        while (my @liste_new_mpi = $sth->fetchrow_array) {
            print "$liste_new_mpi[0] $liste_new_mpi[1]\n";
        }

        # Reordonnement de la table mpi
        # -----------------------------
        $dbh->do("ALTER TABLE `mpi` ORDER BY `nbprocs` ");

        $rc = $sth->finish;
        
        # Affichage d'un warning pour mettre a jour la couleur
        # ----------------------------------------------------
        print "\n";
        print "ATTENTION !!!!!\n";
        print "NE PAS OUBLIER DE MODIFIER A LA MAIN LE CHAMP ";
        print "couleur POUR CHAQUE NOUVELLE ENTREE\n";
    }

    return ($nbmpi_new);
}


sub update_maui_table {
    my ($p2mconfig,$dbh,$nextmon,$veille,$servername) = @_;
    
    #######################################################################################
    # Mise a jour de la table MAUI pour toutes les dates depuis le dernier enregistrement #
    #######################################################################################
    
    # Nombre d'enregistrements dans la table avant mise a jour
    # --------------------------------------------------------
    my $sth = $dbh->prepare("select * from maui")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $rc = $sth->finish;
    my $nbmaui_old = $sth->rows;
    
    # Ajout des nouveaux enregistrements
    # ----------------------------------
    print "\n";
    print "Mise a jour de la table maui\n";
    print "----------------------------\n";
    for ( my $newmon = $nextmon ; &Date_Cmp($newmon,$veille) <= 0 ; $newmon = &DateCalc($newmon,"+ 1days") ) {
    
    # Fichier de log de MAUI a exploiter
    # ----------------------------------
    my $mauimon  = &UnixDate($newmon,"%a_%b_%d_%Y"); 
    my $inputfile = "$p2mconfig->{pathmaui}/$mauimon";
    print "Logfile MAUI = $inputfile\n";
    print "\n";
    
    open (FILEIN,"<$inputfile");
    map { chomp } my @lines = <FILEIN>;
    close FILEIN;
    
    foreach my $line (@lines) {
        if ($line !~ /^VERSION.*$/ && $line !~ /^#.*$/) {
            my @champs = split(/\s+/, $line);
            my $jobid = $champs[0];
            my $status = $champs[6];
            my $classe = $champs[7];
            my $submission = $champs[8];
            my $dispatch = $champs[9];
            my $start = $champs[10];
            my $completion = $champs[11];
            my $account = $champs[25];
            my $tasks = $champs[21];
            my $procspertask = $champs[31];
        #      my $hosts = $champs[37];
            my $partition = $champs[30];
        
        #      print "$ident;$jobid;$date;$status;$classe;\n";
        #      print "$submission;$dispatch;$start;$completion;\n";
        #      print "$account;$tasks;$procspertask;$hosts;$partition\n";
        
            $sth = $dbh->prepare("select ident,date from pbs where jobid = $jobid and qtime = $submission and servername like '$servername' ")    
            or die "Couldn't prepare statement: " . $dbh->errstr;
            $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
            my $ident = 0;
            my $date = "";
            while (my @result = $sth->fetchrow_array) {
                $ident = $result[0];
                $date = $result[1];
            }
            $rc = $sth->finish;
        #     print "$ident $jobid $date $servername $submission\n";
            
        #     Verification correspondance jobs avec table pbs
        #     -----------------------------------------------
            if ($ident == 0) {
                $date = &UnixDate($newmon,"%Y-%m-%d");
                print "ATTENTION !!!!!\n";
                print "Aucun job equivalent dans la table pbs\n";
                print "ident = $ident jobid = $jobid date = $date\n";
                print "\n";
            }
        
        #     Verification coherence dates
        #     ----------------------------
            my $date_completion  = &ParseDate("epoch $completion");
            my $heure_completion = &UnixDate($date_completion,"%T");
            $date_completion = &UnixDate($date_completion,"%Y-%m-%d");
            if ($date_completion ne $date) {
                print "ATTENTION !!!!!\n";
                print "Incoherence sur les dates dans la table maui\n";
                print "ident = $ident jobid = $jobid date = $date date_completion = $date_completion\n";
                print "heure_completion = $heure_completion et completion-start = ".($completion-$start)."\n";
                print "\n";
            }
        
        #     Identification jobs grille des users locaux et jobs MPI
        #     -------------------------------------------------------
        #    if ($account ne "[NONE]" || $tasks > 1) {
        #        print "Le job $jobid de la classe $classe a tourne sur $tasks procs avec l'account $account\n";
        #        print "\n";
        #    }   
            
            #print "$ident $jobid $date $status $classe $start $completion $account $tasks $procspertask $partition\n";

            $dbh->do("insert into maui (ident,jobid,date,status,classe,submission,dispatch,start,completion,
                                        account,tasks,procspertask,partition) values
                    ('$ident','$jobid','$date','$status','$classe','$submission','$dispatch','$start','$completion',
                    '$account','$tasks','$procspertask','$partition') ");

            $dbh->do("update pbsmaui set account='$account' where ident='$ident' ");
            }
        }

        # Identification nouveaux jobs grille des users locaux et jobs MPI
        # ----------------------------------------------------------------
        my $newmon_ymd = &UnixDate($newmon,"%Y-%m-%d");
        my $newmon_dmy = &UnixDate($newmon,"%d-%m-%Y");
        $sth = $dbh->prepare("select count(jobid),classe,tasks,account from maui where date like '$newmon_ymd'
                              group by classe,tasks,account")
        or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
        my $tmpnbjobs = 0;
        my $tmpclasse = "";
        my $tmptasks = 0;
        my $tmpaccount = "";
        while (my @result = $sth->fetchrow_array) {
            $tmpnbjobs  = $result[0];
            $tmpclasse  = $result[1];
            $tmptasks   = $result[2];
            $tmpaccount = $result[3];
            if ($tmpaccount ne "[NONE]" || $tmptasks > 1) {
                print "Le $newmon_dmy, $tmpnbjobs jobs de la classe $tmpclasse ont tourne sur $tmptasks procs avec l'account $tmpaccount\n";
            }
        }
        $rc = $sth->finish;    
        print "\n";
    }

    # Nombre d'enregistrements dans la table apres mise a jour
    # --------------------------------------------------------
    $sth = $dbh->prepare("select * from maui")
    or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
    or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbmaui_new = $sth->rows;
    
    # Affichage du nombre de nouveaux enregistrements
    # -----------------------------------------------
    my $nbmaui_delta = $nbmaui_new - $nbmaui_old;
    print "$nbmaui_delta enregistrements ont ete ajoutes\n";
    print "\n";
    
    return ($nbmaui_delta);
}


sub update_classes_table {
    my ($dbh,$nextmon,$veille) = @_;
    
    ###############################################
    # Mise a jour de la table des classes de MAUI #
    ###############################################
    
    # Nombre de classes dans la table avant mise a jour
    # -------------------------------------------------
    my $sth = $dbh->prepare("select * from classes")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $rc = $sth->finish;
    my $nbclasse_old = $sth->rows;
    
    # Ajout des nouvelles classes
    # ---------------------------
    print "Mise a jour de la table classes\n";
    print "-------------------------------\n";
    $sth = $dbh->prepare("select classe from maui where date between '"
                         .ymd_date($nextmon)."' and '".ymd_date($veille)."' group by classe")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    while (my @liste_classe = $sth->fetchrow_array) {
        #  print "classe = $liste_classe[0]\n";
        $dbh->do("insert into classes (classe) values ('$liste_classe[0]')
                  on duplicate key update classe='$liste_classe[0]'");
    }
    $rc = $sth->finish;
    
    # Nombre de classes dans la table apres mise a jour
    # -------------------------------------------------
    $sth = $dbh->prepare("select * from classes")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    $rc = $sth->finish;
    my $nbclasse_new = $sth->rows;
    
    # Affichage du nombre de nouvelles classes
    # ----------------------------------------
    my $nbclasse_delta = $nbclasse_new - $nbclasse_old;
    print "$nbclasse_delta classes ont ete ajoutees\n";
    
    # Affichage du nom des nouvelles classes
    # --------------------------------------
    if ($nbclasse_delta != 0) {
        my $nbclasse_limit = $nbclasse_new-1;
        $sth = $dbh->prepare("select classe from maui
                group by classe order by date asc
                limit $nbclasse_old,$nbclasse_limit")
            or die "Couldn't prepare statement: " . $dbh->errstr;
        $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        while (my @liste_new_classe = $sth->fetchrow_array) {
            print "$liste_new_classe[0]\n";
        }
        $rc = $sth->finish;
    }

    return ($nbclasse_new);
}


sub update_pbs_month_table {
    my ($dbh,$nextmon,$veille) = @_;
    my $nbpbsmois_delta = 0;

    ##########################################################################################################
    # Mise a jour de la table des infos PBS mensuelles apres le (n-1)eme jour du mois pour le mois precedent #
    ##########################################################################################################
        
    # Calcul du dernier mois d'enregistrement
    #  --------------------------------------
    my $sth = $dbh->prepare("select max(yearmonth) from pbsmois")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    my $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    
    my $last_yearmonth = $sth->fetchrow_array;
    my $rc = $sth->finish;
    print "\n";
    print "Dernier mois d'enregistrement = $last_yearmonth\n";
    
    # Calcul du premier jour du dernier mois d'enregistrement
    #  ------------------------------------------------------
    my $first_day_of_last_yearmonthq = $last_yearmonth."-01";
    print "Premier jour du dernier mois d'enregistrement = $first_day_of_last_yearmonthq\n";
    
    # Calcul du premier jour du futur mois d'enregistrement
    #  ----------------------------------------------------
    my $first_day_of_next_yearmonth  = &DateCalc($first_day_of_last_yearmonthq,"+1 months");
    my $first_day_of_next_yearmonthq = &UnixDate($first_day_of_next_yearmonth,"%Y-%m-%d");
    print "Premier jour du futur mois d'enregistrement = $first_day_of_next_yearmonthq\n";
    
    # Calcul du nombre maximal de jours d'execution pour un job depuis le dernier mois enregistre
    # -------------------------------------------------------------------------------------------
    $sth = $dbh->prepare("select (2+floor(max(walltime-time_to_sec(heure))/86400)) from pbs
                        where date between '$first_day_of_next_yearmonthq' and '"
                        .ymd_date($veille)."' ")
        or die "Couldn't prepare statement: " . $dbh->errstr;
    $rv = $sth->execute
        or die "Couldn't execute statement: " . $sth->errstr;
    my $max_run_days = $sth->fetchrow_array;
    $rc = $sth->finish;
    print "\n";
    print "Nb maximal de jours d'execution pour un job depuis le dernier mois enregistre = $max_run_days\n";
    
    print "\n";
    print "Mise a jour de la table pbsmois\n";
    print "-------------------------------\n";
        
    for ( my $newmon = $nextmon ; &Date_Cmp($newmon,$veille) <= 0 ; $newmon = &DateCalc($newmon,"+ 1days") ) {
        my $newmonq = &UnixDate($newmon,"%Y-%m-%d");
                        
        # Recuperation des donnees apres le (n-1)eme jour du mois pour etre
        # sur que tous les jobs commences le mois precedent sont termines
        # -----------------------------------------------------------------
        my $currday = &UnixDate($newmon,"%e");
        
        if ($currday == $max_run_days-1) {
        
            my $currdaym1 = $currday - 1;
        
            my $day1lastmonth = &DateCalc($newmonq,"-1 months -$currdaym1 days");
            my $day1lastmonthq = &UnixDate($day1lastmonth,"%Y-%m-%d");
        
            my $daynlastmonth  = &DateCalc($newmonq,"-$currday days");
            my $daynlastmonthq = &UnixDate($daynlastmonth,"%Y-%m-%d");
        
            my $daymaxcurrmonth  = &DateCalc($daynlastmonth,"+$currday days");
            my $daymaxcurrmonthq = &UnixDate($daymaxcurrmonth,"%Y-%m-%d");
        
            print "\n";
            print "Nouveau mois enregistre $day1lastmonthq -> $daynlastmonthq\n";
            print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n";
            my $yearmonth = &UnixDate($daynlastmonth,"%Y-%m");
        
        #   Nombre de jours du mois
        #   -----------------------
            my $nbdays = &DateCalc($day1lastmonth,$daynlastmonth);
            my @tab = split(/:/,$nbdays);
            $nbdays = 7*$tab[2]+$tab[3]+1;
            print "$nbdays jours dans le mois\n";
        
        #   Liste de tous les utilisateurs ayant utilise des ressources pendant le mois
        #   ---------------------------------------------------------------------------
            print "\n$day1lastmonthq -> $daymaxcurrmonthq\n";
            print "~~~~~~~~~~~~~~~~~~~~~~~~\n";
        
            $sth = $dbh->prepare("select user,pbs.groupe,groupes.queue,walltimesplit,
                nbprocs,DATEDIFF(date,'$day1lastmonthq')
                from pbs join groupes on pbs.groupe = groupes.groupe
                where date between '$day1lastmonthq' and '$daymaxcurrmonthq'
                order by user,pbs.groupe")
            or die "Couldn't prepare statement: " . $dbh->errstr;
            $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        
            my $user   = "";
            my $groupe = "";
            my $queue  = "";
            my @data   = "";
            my $sumwalltimesplit = "";
        
            while (my @resultat = $sth->fetchrow_array) {
                my @wt = split(/\+/,$resultat[3]);
                my $j = $resultat[5];
                if ($resultat[0] ne $user || $resultat[1] ne $groupe) {
                    if ($user ne "" || $groupe ne "") {
                        $sumwalltimesplit = sum(@data);
                        my $sth2 = $dbh->prepare("select count(user) from pbs
                                    where date between '$day1lastmonthq' and '$daynlastmonthq'
                                    and user like '$user' and groupe like '$groupe' ")
                                or die "Couldn't prepare statement: " . $dbh->errstr;
                        my $rv2 = $sth2->execute
                                or die "Couldn't execute statement: " . $sth2->errstr;
                        my $res2 = $sth2->fetchrow_array;
                        my $rc2 = $sth2->finish;
                        if ($res2 != 0 || $sumwalltimesplit ne "0") {
                            print "$yearmonth $user $groupe $queue $sumwalltimesplit\n";
                            $dbh->do("insert into pbsmois (yearmonth,user,groupe,queue,sumwalltimesplit)
                                    values ('$yearmonth','$user','$groupe','$queue','$sumwalltimesplit') ");
                            $nbpbsmois_delta += 1;
                        }
                    }
                    $user   = $resultat[0];
                    $groupe = $resultat[1];
                    $queue  = $resultat[2];
                    @data   = 0;
                }
                for (my $i=0;$i<=$#wt;$i++) {
                    my $k = $j-$i;
                    if (($k >= 0) && ($k < $nbdays)) {
                        $data[$i] += ($wt[$i] * $resultat[4]);
                    }
                }
            }
            $rc = $sth->finish;
        
            $sumwalltimesplit = sum(@data);
            my $sth2 = $dbh->prepare("select count(user) from pbs
                    where date between '$day1lastmonthq' and '$daynlastmonthq'
                    and user like '$user' ")
                    or die "Couldn't prepare statement: " . $dbh->errstr;
            my $rv2 = $sth2->execute
                    or die "Couldn't execute statement: " . $sth2->errstr;
            my $res2 = $sth2->fetchrow_array;
            my $rc2 = $sth2->finish;
            if ($res2 != 0 || $sumwalltimesplit ne "0") {
            print "$yearmonth $user $groupe $queue $sumwalltimesplit\n";
            $dbh->do("insert into pbsmois (yearmonth,user,groupe,queue,sumwalltimesplit)
                        values ('$yearmonth','$user','$groupe','$queue','$sumwalltimesplit') ");
        
            $nbpbsmois_delta += 1;
            print "\n";
            print "$nbpbsmois_delta enregistrements ont ete ajoutes\n";
            }
        
        #   Nombre de jobs et temps cumules par utilisateur pour le mois ecoule
        #   -------------------------------------------------------------------
            print "\n$day1lastmonthq -> $daynlastmonthq\n";
            print "~~~~~~~~~~~~~~~~~~~~~~~~\n";
        
            $sth = $dbh->prepare("select user,groupe,count(user),sum(starttime-etime),
                sum(walltime),sum(cputime/nbprocs) from pbs
                where date between '$day1lastmonthq' and '$daynlastmonthq'
                group by user,groupe")
            or die "Couldn't prepare statement: " . $dbh->errstr;
            $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        
            my $user0       = "";
            my $groupe0     = "";
            my $nbjobs      = "0";
            my $sumqtime    = "0";
            my $sumwalltime = "0";
            my $sumcputime  = "0";
        
            while (my @resultat0 = $sth->fetchrow_array) {
            $user0       = $resultat0[0];
            $groupe0     = $resultat0[1];
            $nbjobs      = $resultat0[2];
            $sumqtime    = $resultat0[3];
            $sumwalltime = $resultat0[4];
            $sumcputime  = $resultat0[5];
        
            print "$user0 $groupe0 $nbjobs $sumqtime $sumwalltime $sumcputime\n";
            $dbh->do("update pbsmois set nbjobs=$nbjobs,sumqtime=$sumqtime,
                        sumwalltime=$sumwalltime,sumcputime=$sumcputime
                        where yearmonth='$yearmonth' and user='$user0' and groupe='$groupe0' ");
            }
            $rc = $sth->finish;
            print "\n";
        
        # Si on ne recupere pas les donnees du (n-1)eme jour du mois
        # ----------------------------------------------------------
        } else {
            print "Pas de nouvel enregistrement mensuel\n";
        }
    }
    return ($nbpbsmois_delta,$max_run_days);
}


sub update_mpi_month_table {
    my ($dbh,$nextmon,$veille,$max_run_days) = @_;
    my $nbmpimois_delta = 0;

    ##########################################################################################################
    # Mise a jour de la table des infos MPI mensuelles apres le (n-1)eme jour du mois pour le mois precedent #
    ##########################################################################################################
    
    print "\n";
    print "Mise a jour de la table mpimois\n";
    print "-------------------------------\n";
    
    for ( my $newmon = $nextmon ; &Date_Cmp($newmon,$veille) <= 0 ; $newmon = &DateCalc($newmon,"+ 1days") ) {
        my $newmonq = &UnixDate($newmon,"%Y-%m-%d");
        
        # Recuperation des donnees apres le (n-1)eme jour du mois pour etre
        # sur que tous les jobs commences le mois precedent sont termines
        # -----------------------------------------------------------------
        my $currday = &UnixDate($newmon,"%e");
        
        if ($currday == $max_run_days-1) {
        
            my $currdaym1 = $currday - 1;
        
            my $day1lastmonth = &DateCalc($newmonq,"-1 months -$currdaym1 days");
            my $day1lastmonthq = &UnixDate($day1lastmonth,"%Y-%m-%d");
        
            my $daynlastmonth  = &DateCalc($newmonq,"-$currday days");
            my $daynlastmonthq = &UnixDate($daynlastmonth,"%Y-%m-%d");
        
            my $daymaxcurrmonth  = &DateCalc($daynlastmonth,"+$currday days");
            my $daymaxcurrmonthq = &UnixDate($daymaxcurrmonth,"%Y-%m-%d");
        
            print "\n";
            print "Nouveau mois enregistre $day1lastmonthq -> $daynlastmonthq\n";
            print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n";
            my $yearmonth = &UnixDate($daynlastmonth,"%Y-%m");
        
        #   Nombre de jours concernes
        #   -------------------------
            my $nbdays = &DateCalc($day1lastmonth,$daynlastmonth);
            my @tab = split(/:/,$nbdays);
            $nbdays = 7*$tab[2]+$tab[3]+1;
            print "$nbdays jours dans le mois\n";
        
        #   Liste de tous les valeurs de nbprocs et groupes ayant utilise des ressources pendant le mois
        #   --------------------------------------------------------------------------------------------
            print "\n$day1lastmonthq -> $daymaxcurrmonthq\n";
            print "~~~~~~~~~~~~~~~~~~~~~~~~\n";
        
            my $sth = $dbh->prepare("select nbprocs,groupe,walltimesplit,
                DATEDIFF(date,'$day1lastmonthq') from pbs
                where date between '$day1lastmonthq' and '$daymaxcurrmonthq'
                order by nbprocs,groupe")
            or die "Couldn't prepare statement: " . $dbh->errstr;
            my $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        
            my $nbprocs = 0;
            my $groupe  = "";
            my @data    = "";
            my $sumwalltimesplit = "";
        
            while (my @resultat = $sth->fetchrow_array) {
                my @wt = split(/\+/,$resultat[2]);
                my $j = $resultat[3];
                if ($resultat[0] ne $nbprocs || $resultat[1] ne $groupe) {
                    if ($nbprocs != 0) {
                        $sumwalltimesplit = sum(@data);
                        my $sth2 = $dbh->prepare("select count(nbprocs) from pbs
                                    where date between '$day1lastmonthq' and '$daynlastmonthq'
                                    and nbprocs like '$nbprocs' and groupe like '$groupe' ")
                                or die "Couldn't prepare statement: " . $dbh->errstr;
                        my $rv2 = $sth2->execute
                                or die "Couldn't execute statement: " . $sth2->errstr;
                        my $res2 = $sth2->fetchrow_array;
                        my $rc2 = $sth2->finish;
                        if ($res2 != 0 || $sumwalltimesplit ne "0") {
                            print "$yearmonth $nbprocs $sumwalltimesplit\n";
                            $dbh->do("insert into mpimois (yearmonth,nbprocs,groupe,sumwalltimesplit)
                                    values ('$yearmonth','$nbprocs','$groupe','$sumwalltimesplit') ");
                            $nbmpimois_delta += 1;
                        }
                    }
                    $nbprocs = $resultat[0];
                    $groupe  = $resultat[1];
                    @data    = 0;
                }
                for (my $i=0;$i<=$#wt;$i++) {
                    my $k = $j-$i;
                    if (($k >= 0) && ($k < $nbdays)) {
                    $data[$i] += ($wt[$i] * $resultat[0]);
                    }
                }
            }
            my $rc = $sth->finish;
        
            $sumwalltimesplit = sum(@data);
            my $sth2 = $dbh->prepare("select count(jobid) from pbs
                    where date between '$day1lastmonthq' and '$daynlastmonthq'
                    and nbprocs like '$nbprocs' and groupe like '$groupe' ")
                    or die "Couldn't prepare statement: " . $dbh->errstr;
            my $rv2 = $sth2->execute
                    or die "Couldn't execute statement: " . $sth2->errstr;
            my $res2 = $sth2->fetchrow_array;
            my $rc2 = $sth2->finish;
            if ($res2 != 0 || $sumwalltimesplit ne "0") {
                print "$yearmonth $nbprocs $sumwalltimesplit\n";
                $dbh->do("insert into mpimois (yearmonth,nbprocs,groupe,sumwalltimesplit)
                            values ('$yearmonth','$nbprocs','$groupe','$sumwalltimesplit') ");
            
                $nbmpimois_delta += 1;
                print "\n";
                print "$nbmpimois_delta enregistrements ont ete ajoutes\n";
            }
        
        #   Nombre de jobs et temps cumules par valeur de nbprocs et par groupe pour le mois ecoule
        #   ---------------------------------------------------------------------------------------
            print "\n$day1lastmonthq -> $daynlastmonthq\n";
            print "~~~~~~~~~~~~~~~~~~~~~~~~\n";
        
            $sth = $dbh->prepare("select nbprocs,groupe,count(jobid),sum(starttime-etime),
                sum(walltime),sum(cputime/nbprocs) from pbs
                where date between '$day1lastmonthq' and '$daynlastmonthq'
                group by nbprocs,groupe")
            or die "Couldn't prepare statement: " . $dbh->errstr;
            $rv = $sth->execute
            or die "Couldn't execute statement: " . $sth->errstr;
        
            my $nbprocs0    = "";
            my $groupe0     = "";
            my $nbjobs      = "0";
            my $sumqtime    = "0";
            my $sumwalltime = "0";
            my $sumcputime  = "0";
        
            while (my @resultat0 = $sth->fetchrow_array) {
                $nbprocs0    = $resultat0[0];
                $groupe0     = $resultat0[1];
                $nbjobs      = $resultat0[2];
                $sumqtime    = $resultat0[3];
                $sumwalltime = $resultat0[4];
                $sumcputime  = $resultat0[5];
            
                print "$nbprocs0 $groupe0 $nbjobs $sumqtime $sumwalltime $sumcputime\n";
                $dbh->do("update mpimois set nbjobs=$nbjobs,sumqtime=$sumqtime,
                            sumwalltime=$sumwalltime,sumcputime=$sumcputime
                            where yearmonth='$yearmonth' and nbprocs='$nbprocs0' and groupe='$groupe0'");
            }
            $rc = $sth->finish;
            print "\n";
        
        # Si on ne recupere pas les donnees du (n-1)eme jour du mois
        # ----------------------------------------------------------
        } else {
            print "Pas de nouvel enregistrement mensuel\n";
        }
    }

    return ($nbmpimois_delta);
}


# ---------------------------------------------------------------------------------
# Main subroutine
# ---------------------------------------------------------------------------------
sub main {
    #################################################################
    # Affichage de la date d'execution de ce script en debut de log #
    #################################################################
    my $aujourdhui  = ParseDate("now");
    my $aujourdhuiq = UnixDate($aujourdhui,"%d-%m-%Y %H:%M:%S");
    print "#############################################################################\n";
    print "# Date et heure d'enregistrement des logs TORQUE/MAUI : $aujourdhuiq #\n";
    print "#############################################################################\n";
    print "\n";
    my $p2mconfig=read_config_file();
    my $dbh=open_database($p2mconfig);    
    
    # Identification du serveur TORQUE/MAUI
    # -------------------------------------
    map { chomp } my $servername = `/bin/hostname`;
    print "Nom du serveur TORQUE/MAUI = $servername\n";


    
    # Determination de la veille du jour courant
    # ------------------------------------------
    my $veille = ParseDate("yesterday at midnight");
    
    my $firstmon=get_first_entry_date($p2mconfig,$dbh);
    print "Premier jour d'enregistrement  = ".ymd_date($firstmon)."\n";
    my $lastmon=get_last_entry_date  ($dbh,$firstmon);
    print "Dernier jour d'enregistrement  = ".ymd_date($lastmon)."\n";
    my $nextmon = DateCalc($lastmon,"+ 1days");
    
    my $dummy; # Dummy value, just aesthetic
    my ($nbcalcul,$nbpbs,$nbgroupes,$nbroles,$nbusers,$nbmpi,$nbmaui,$nbclasses,$max_run_days);
    #my ($nbcalcul,$nbpbs,$nbgroupes,$nbusers,$nbmpi,$nbmaui,$nbclasses,$max_run_days);
    my ($nbpbsmois,$nbmpimois);
    
    $nbcalcul    =update_calcul_table       (           $dbh,$nextmon,$veille,$servername  );
    $nbpbs       =update_pbs_table          ($p2mconfig,$dbh,$nextmon,$veille              );
    #$nbgroupes   =update_pbs_group_table    (           $dbh,$nextmon,$veille              );
    $nbgroupes   =update_groupes_table      (           $dbh,$nextmon,$veille              );
    ($nbroles,$nbusers)     =update_user_and_role_table(           $dbh,$nextmon,$veille              );
    #$nbusers     =update_users_table        (           $dbh,$nextmon,$veille              );
    $nbmpi       =update_mpi_table          (           $dbh,$nextmon,$veille              );
    $nbmaui      =update_maui_table         ($p2mconfig,$dbh,$nextmon,$veille,$servername  );
    $nbclasses   =update_classes_table      (           $dbh,$nextmon,$veille              );
    ($nbpbsmois,$max_run_days)=update_pbs_month_table    (           $dbh,$nextmon,$veille              );
    $nbmpimois       =update_mpi_month_table    (           $dbh,$nextmon,$veille,$max_run_days);
 
    
    # Coherence entre nouveaux enregistrements de pbs et de maui
    # ----------------------------------------------------------
    if ($nbmaui != $nbpbs) {
        print "ATTENTION !!!!!\n";
        print "Incoherence entre les table pbs et maui\n";
        print "Ecart = ".abs($nbpbs-$nbmaui)." enregistrements\n";
        print "\n"; 
    }

    # Nombre d'elements dans les tables
    # ---------------------------------
    print "\n";
    print "Nombre d'enregistrements\n";
    print "------------------------\n";
    print "Table calcul  : $nbcalcul\n";
    print "Table classes : $nbclasses\n";
    print "Table groupes : $nbgroupes\n";
    print "Table maui    : + $nbmaui\n";
    print "Table mpi     : $nbmpi\n";
    print "Table mpimois : + $nbmpimois\n";
    print "Table pbs     : + $nbpbs\n";
    print "Table pbsmois : + $nbpbsmois\n";
    #print "Table roles   : $nbroles\n";
    print "Table users   : $nbusers\n";
    
    ##########################
    # Deconnection de la BDD #
    ##########################
    print "\n";
    print "Deconnection de la BDD\n";
    print "----------------------\n";
    print "\n";
    my $rc = $dbh->disconnect();
}


# ---------------------------------------------------------------------------------
# Calling main()
# ---------------------------------------------------------------------------------
&main();
# ---------------------------------------------------------------------------------






