<?php

/****************************************************************************//**
 *
 * @file		creationimagesaccounting.php
 * @brief 		Fichier de création des images pour les pages fixes de l'accounting
 *
 * creationimagesaccounting.php permet de créer des graphes concernant l'utilisation de la 
 * grille en ligne de commande.
 * Les graphes permettent le suivi de l'occupation des ressources, du nombre de jobs terminés,
 * des Durées moyennes des jobs, ...
 *
 *********************************************************************************/

/****************************************************************************//**
 *
 * Déclation des INCLUDES
 *
 *********************************************************************************/

include	("./inc/accparam.inc.php"); 		// Lecture des paramètres d'environnement
include ("./inc/accexception.inc.php"); 	// Gestion des exceptions
include ("./inc/accgraphparam.inc.php");	// Paramètres des graphes
include ("./inc/accdates.inc.php"); 		// Gestion des dates
include ("./inc/accsql.inc.php");			// Gestion SQL
include ("./inc/accsqliphc.inc.php");		// Gestion SQL
include ("./inc/accsqllapp.inc.php");		// Gestion SQL
include ("./inc/accgraph.inc.php");   		// Gestion des graphes

/****************************************************************************//**
 *
 * Chargement des valeurs par défaut
 *
 *********************************************************************************/

try
{
	$sellist="";
	// Déclaration des dates pour les graphiques
	$now=time();
	$mdate = new Accdates(date('d-m-Y',$now));
	$date_yesterday = $mdate->getDateYesterday();
	$date_yesterdayYmd = $mdate->getDateYesterdayYmd();
	$date_monthfirstdayYmd  = $mdate->getFirstDayOfMonthYmd();
	$date_yearfirstdayYmd  = $mdate->getFirstDayOfYearYmd();

	// Selection des spécifités du site
	switch ($param_site)
	{
		case "IPHC":     // Gestion spécifique à l'IPHC
			$msql = new accSqliphc($param_mysql_servername,$param_mysql_dbname,$param_mysql_login,$param_mysql_passwd);
			$msql->setSgbdIphc();
			break;
				
		case "LAPP":     // Gestion spécifique au LAPP
			$msql = new accSqllapp($param_mysql_servername,$param_mysql_dbname,$param_mysql_login,$param_mysql_passwd);
			break;
	}
	// Connexion à la base de données
	$msql->verifConnexion();
	// Récupération de la première date dans pbsmaui;
	$date_firstpbsmauiYmd = $msql->getFirstDatePbsmaui();

	/****************************************************************************//**
	 *
	 * MAIN
	 * Partie dédiée à la création des graphes
	 *
	 *********************************************************************************/
	// Définition des paramètres par défaut
	$graph_end = $date_yesterdayYmd;
	$graph_type = "percwalltime";
	$graph_jobs = "all";
	$graph_surcharge = "avec";
	$graph_procs = "vprocs";
	$graph_group ="";

	// Liste des types de graphes
	$graph_type_list[] = "percwalltime";
	$graph_type_list[] = "numwalltime";
	$graph_type_list[] = "daywalltime";
	$graph_type_list[] = "percnbjobs";
	$graph_type_list[] = "numnbjobs";
	$graph_type_list[] = "daynbjobs";
	$graph_type_list[] = "numwalltimenbjobs";
	$graph_type_list[] = "avgtimes";
		
	$nb_graph_type = count($graph_type_list);
		
	for ($i=0;$i<$nb_graph_type;$i++)
	{
		$graph_type=$graph_type_list[$i];
		$graph_option_periode = "day";

		// Gestion des graphes de l'accounting pour les données d'hier
		// Import des paramètres du graphe
		$graph_start = $date_yesterdayYmd;
		$full_graph_filename=$param_abs_images_path."/".$graph_type."_yesterday.png";
		$accparam=new Accgraphparam($graph_type, $graph_start, $graph_end, $graph_option_periode, $graph_jobs, $graph_surcharge, $graph_group, $graph_procs, $full_graph_filename);
		// Récupération des données Mysql
		$data=$msql->getDataGraph($accparam);
		// Création du graphe
		$mongraph=new Accgraph($accparam, $data);
		$mongraph->createGraph();

		// Gestion des graphes de l'accounting pour les données du mois en cours
		$graph_start = $date_monthfirstdayYmd;
		$full_graph_filename=$param_abs_images_path."/".$graph_type."_current_month.png";
		$accparam=new Accgraphparam($graph_type, $graph_start, $graph_end, $graph_option_periode, $graph_jobs, $graph_surcharge, $graph_group, $graph_procs, $full_graph_filename);
		// Récupération des données Mysql
		$data=$msql->getDataGraph($accparam);
		// Création du graphe
		$mongraph=new Accgraph($accparam, $data);
		$mongraph->createGraph();

		// Gestion des graphes de l'accounting pour les données de l'année en cours
		switch ($graph_type)
		{
			case "daywalltime":
				$graph_option_periode = "week";
				break;

			case "daynbjobs":
				$graph_option_periode = "week";
				break;

			default:
				$graph_option_periode = "day";
				break;
		}
		
		$graph_start = $date_yearfirstdayYmd ;
		$full_graph_filename=$param_abs_images_path."/".$graph_type."_current_year.png";
		$accparam=new Accgraphparam($graph_type, $graph_start, $graph_end, $graph_option_periode, $graph_jobs, $graph_surcharge, $graph_group, $graph_procs, $full_graph_filename);
		// Récupération des données Mysql
		$data=$msql->getDataGraph($accparam);
		// Création du graphe
		$mongraph=new Accgraph($accparam, $data);
		$mongraph->createGraph();
		// Gestion des graphes de l'accounting pour toutes les données
		switch ($graph_type)
		{
			case "daywalltime":
				$graph_option_periode = "month";
				break;

			case "daynbjobs":
				$graph_option_periode = "month";
				break;

			default:
				$graph_option_periode = "day";
				break;
		}

		$graph_start = $date_firstpbsmauiYmd;
		$full_graph_filename=$param_abs_images_path."/".$graph_type."_all_date.png";
		$accparam=new Accgraphparam($graph_type, $graph_start, $graph_end, $graph_option_periode, $graph_jobs, $graph_surcharge, $graph_group, $graph_procs, $full_graph_filename);
		// Récupération des données Mysql
		$data=$msql->getDataGraph($accparam);
		// Création du graphe
		$mongraph=new Accgraph($accparam, $data);
		$mongraph->createGraph();
	}
}

catch (accException $Ae) // Gestion des erreurs
{
	$Ae->getExceptionExit();
}
?>

