<?php
/****************************************************************************//**
 * @file	accdates.inc.php
 * @brief	Classe de gestion des dates
 *
 * @author Eric
 * @version 0.5
 *********************************************************************************/

/****************************************************************************//**
 * @brief Classe accDates
 *
 * Classe de gestion des dates
 *
 * @param	public $date;
 *
 *********************************************************************************/

class accDates {
	/****************************************************************************//**
	 * Déclaration des variables
	 ********************************************************************************/
	public $date;

	/****************************************************************************//**
	 * Déclaration des méthodes
	 *********************************************************************************/

	/****************************************************************************//**
	 * @brief Constructeur
	 *
	 * Initialisation de l'objet
	 *
	 * @param $date;
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	function __construct($date)
	{
		if (isset($date))
		{
			$this->date = $date;
		}
		else
		{
			throw new accException("CT_ERR_DATE"); // Erreur date
		}
	}

	/****************************************************************************//**
	 * @brief Affiche la date
	 *
	 *  Affiche la date
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	function show()
	{
		echo "La date : $this->date";
	}

	/****************************************************************************//**
	 * @brief Convertion de format de date
	 *
	 * Convertit le format de la date de jj-mm-aaaa to aaaa-mm-jj
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	function convertDate()
	{
		$tab = explode("-",$this->date);
		$tab = array_reverse($tab);
		$this->date = implode("-", $tab);
	}

	/****************************************************************************//**
	 * @brief Convertion de format de date
	 *
	 * Convertit le format de la date de jj-mm-aaaa to aaaa-mm-jj
	 *
	 * @param 	date
	 * @return	date
	 *********************************************************************************/
	function revertDate($date)
	{
		$tab = explode("-",$date);
		$tab = array_reverse($tab);
		$date = implode("-", $tab);
		return $date;
	}

	/****************************************************************************//**
	 * @brief La date de hier
	 *
	 * Retourne la date de hier au format (d-m-Y)
	 *
	 * @param 	aucun
	 * @return	$date_yesterday
	 *********************************************************************************/
	function getDateYesterday()
	{
		$date_yesterday = date('d-m-Y', time() - 3600 * 24);
		return $date_yesterday;
	}

	/****************************************************************************//**
	 * @brief La date du premier jour du mois
	 *
	 * Retourne la date du premier jour du mois au format (d-m-Y)
	 *
	 * @param 	aucun
	 * @return	$first_day_of_month
	 *********************************************************************************/
	function getFirstDayOfMonth()
	{
		list($Day,$month,$year)=explode("-",$this->date);
		$mkdate= mktime(0, 0, 0, $month, 1, $year);
		$first_day_of_month = strftime("%d-%m-%Y", $mkdate);
		return ($first_day_of_month);
	}

	/****************************************************************************//**
	 * @brief La date du premier jour de l''année
	 *
	 * Retourne la date du premier jour du mois au format (d-m-Y)
	 *
	 * @param 	aucun
	 * @return	$first_day_of_year
	 *********************************************************************************/
	function getFirstDayOfYear()
	{
		list($Day,$month,$year)=explode("-",$this->date);
		$mkdate= mktime(0, 0, 0, 1, 1, $year);
		$first_day_of_year = strftime("%d-%m-%Y", $mkdate);
		return ($first_day_of_year);
	}

	/****************************************************************************//**
	 * @brief La date de hier
	 *
	 * Retourne la date de hier au format (Y-m-d)
	 *
	 * @param 	aucun
	 * @return	$date_yesterday
	 *********************************************************************************/
	function getDateYesterdayYmd()
	{
		return	$this->revertDate($this->getDateYesterday());
	}

	/****************************************************************************//**
	 * @brief La date du premier jour du mois
	 *
	 * Retourne la date du premier jour du mois au format (Y-m-d)
	 *
	 * @param 	aucun
	 * @return	$first_day_of_month
	 *********************************************************************************/
	function getFirstDayOfMonthYmd()
	{
		return	$this->revertDate($this->getFirstDayOfMonth());
	}

	/****************************************************************************//**
	 * @brief La date du premier jour de l''année
	 *
	 * Retourne la date du premier jour du mois au format (Y-m-d)
	 *
	 * @param 	aucun
	 * @return	$first_day_of_year
	 *********************************************************************************/
	function getFirstDayOfYearYmd()
	{
		return	$this->revertDate($this->getFirstDayOfYear());
	}
}


?>