<?php
/****************************************************************************//**
 * @file	accgraph.inc.php
 * @brief	Classe de création des graphes
 *
 *
 * @author Eric
 * @version 0.8.1
 *********************************************************************************/

/****************************************************************************//**
 * Déclaration des include
 *********************************************************************************/
// Définition du répertoire de stockage de la fonte verdana.ttf
DEFINE("TTF_DIR","./fonts/");
// includes JpGraph
include ("$param_jpgraph_path/jpgraph.php");
include ("$param_jpgraph_path/jpgraph_bar.php");
include ("$param_jpgraph_path/jpgraph_pie.php");
include ("$param_jpgraph_path/jpgraph_pie3d.php");
include ("$param_jpgraph_path/jpgraph_line.php");
include ("$param_jpgraph_path/jpgraph_date.php");
include ("$param_jpgraph_path/jpgraph_log.php" );
include ("$param_jpgraph_path/jpgraph_canvas.php");
include ("$param_jpgraph_path/jpgraph_scatter.php");

include_once ("./inc/accgraphparam.inc.php");

/****************************************************************************//**
 * @brief Classe de création des graphes
 *
 * Classe de gestion des paramètres des graphes
 *
 * @param	var $graphvh; 	-> Les paramètres du graphe
 * @param	var $datavh;	-> Les données sql du graphe
 *********************************************************************************/

class accGraph {
	/****************************************************************************//**
	 * Déclaration des variables
	 *********************************************************************************/
	var $graphvh;
	var $datavh;

	/****************************************************************************//**
	 * Déclaration des méthodes
	 *********************************************************************************/

	/****************************************************************************//**
	 * @brief Constructeur
	 *
	 * Initialisation de l'objet
	 *
	 * @param	var $graphvh; 	-> Les paramètres du graphe
	 * @param	var $datavh;	-> Les données sql du graphe
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	function __construct($pgraphvh, $pdatavh)
	{
		if (isset($pgraphvh))
		{
			if (isset($pdatavh))
			{
				$this->graphvh=$pgraphvh;
				$this->datavh=$pdatavh;
			}
			else
			throw new accException("CT_ERR_DATA");  // Erreur graph data
		}
		else
		throw new accException("CT_ERR_GRAPH_PARAM");  // Erreur graph name
	}

	/****************************************************************************//**
	 * @brief Destructeur
	 *
	 * Destruction de l'objet
	 *********************************************************************************/
	function __destruct()
	{
		$this->graphvh="";
		$this->datavh="";
	}

	/****************************************************************************//**
	 * @brief Construction du graphe
	 *
	 * Concstruction du graphe en fonction du type de graphe
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	function createGraph()
	{
		switch ($this->graphvh->getGraphType())
		{
			case "percwalltime": // Usage des ressources en pourcentage (% walltime)
				$img = $this->makePercGraph();
				break;

			case "percwalltimewa": // Usage des ressources sans le temps disponible en pourcentage (% walltime)
				$img = $this->makePercGraph();
				break;
					
			case "numcputime":  // Usage des ressources en valeurs absolues (cputime)
				$img = $this->makeNumtimeGraph();
				break;

			case "numwalltime":  // Usage des ressources en valeurs absolues (walltime)
				$img = $this->makeNumtimeGraph();
				break;

			case "numwalltimewa":  // Usage des ressources sans le temps disponible en valeurs absolues (walltime)
				$img = $this->makeNumtimeGraph();
				break;

			case "daywalltime":  // Occupation des ressources par jour et par groupe
				$img = $this->makeDayGraph();
				break;
				 
			case "percnbjobs":   // Nombre de jobs en pourcentage (%)
				$img = $this->makePercGraph();
				break;

			case "numnbjobs":   // Nombre de jobs en valeurs absolues
				$img = $this->makeNumGraph();
				break;

			case "daynbjobs":   // Nombre de jobs en valeurs absolues par jour
				$img = $this->makeDayGraph();
				break;

			case "numwalltimenbjobs":   // Nombre de jobs terminés par jour et durées d'occupation
				$img = $this->makeNumtimeGraph();
				break;

			case "avgtimes":     // Durée moyenne des jobs terminés par groupe
				$img = $this->makeNumtimeGraph();
				break;
				 
			case "avgmemjobs":     // Mémoire moyenne et maximale utilisée par les jobs terminés
				$data = $this->makeNumtimeGraph($gvh);
				break;
					
			case "realmemjobs":     // Distribution de mémoire réelle utilisée par les jobs terminés
				$data = $this->makeNumtimeGraph($gvh);
				break;
					
			case "virtualmemjobs":     // Distribution de mémoire virtuelle utilisée par les jobs terminés
				$data = $this->makeNumtimeGraph($gvh);
				break;
					
			case "efficiencyjobs":     // Efficacité des jobs terminés
				$data = $this->makeNumtimeGraph($gvh);
				break;
					
			case "testjpgraph":     // Jpgraph test
				$img = $this->testJpGraph();
				break;
		}
		return($img);
	}

	/****************************************************************************//**
	 * @brief Titre du graphe
	 *
	 * Renvoi le titre à afficher sur le graphe
	 *
	 * @param 	aucun
	 * @return	$graphtitle
	 *********************************************************************************/
	function getGraphTitle()
	{
		switch ($this->graphvh->getGraphType())
		{
			case "percwalltime" :
				$graphtitle = "Occupation des ressources en % par groupe entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "percwalltimewa" :
				$graphtitle = "Occupation des ressources (sans le temps libre) en % par groupe entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "numcputime":
				$graphtitle = "Occupation des ressources en cputime par groupe entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "numwalltime" :
				$graphtitle = "Occupation des ressources en heure par groupe entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "numwalltimewa" :
				$graphtitle = "Occupation des ressources en heure par groupe (sans le temps libre) entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "daywalltime":
				$graphtitle = "Occupation des ressources par jour et par groupe entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "percnbjobs" :
				$graphtitle = "Nombre de jobs terminés par groupe en % entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "numnbjobs" :
				$graphtitle = "Nombre de jobs terminés par groupe en valeur entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "daynbjobs" :
				$graphtitle = "Nombre de jobs terminés par groupe par jour entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "numwalltimenbjobs":   
				$graphtitle = "Nombre de jobs terminés par jour et durées d'occupation entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "avgtimes":    
				$graphtitle = "Durée moyenne des jobs terminés par groupe  entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "avgmemjobs":
				$graphtitle = "Mémoire (réelle+virtuelle) moyenne et maximale utilisée par les jobs entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "realmemjobs":
				$graphtitle = "Mémoire réelle moyenne utilisée par les jobs terminés entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "virtualmemjobs":
				$graphtitle = "Mémoire virtuelle moyenne utilisée par les jobs terminés entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
			case "efficiencyjobs":
				$graphtitle = "Efficacité (cputime/walltime) en % des jobs entre le " .$this->graphvh->getDateStart() ." et le ".$this->graphvh->getDateEnd();
				break;
		}
		return $graphtitle;
	}

	/****************************************************************************//**
	 * @brief Définition des dimension du graphe
	 *
	 * Définition des dimensions de l'image du graphe pour l'ensemble des types de graphe
	 *
	 * @param 	aucun
	 * @return	$dim
	 *********************************************************************************/
	function getGraphDimension()
	{
		$gtype=$this->graphvh->getGraphType();
		if (substr_count ($gtype, "perc") > 0 || substr_count ($gtype, "num")  > 0)
		{
			$dim[] = 900; //495;
			$dim[] = 600; //360;
		}
		else if (substr_count ($gtype, "day") > 0)
		{
			$dim[] = 900; //1000;
			$dim[] = 600; //720;
		}
		else if ($gtype == "avgtimes")
		{
			$dim[] = 900; //1000;
			$dim[] = 440; //660;
		}
		else
		{
			$dim[] = 900;
			$dim[] = 440;
		}
		return($dim);
	}

	/****************************************************************************//**
	 * @brief Dimension X
	 *
	 * Renvoi la dimension X d'un graphe
	 *
	 * @param 	aucun
	 * @return	$dim
	 *********************************************************************************/
	function getGraphDimensionX()
	{
		$dim=$this->getGraphDimension();
		return $dim[0];
	}

	/****************************************************************************//**
	 * @brief Dimension Y
	 *
	 * Renvoi la dimension Y d'un graphe
	 *
	 * @param 	aucun
	 * @return	$dim Y
	 *********************************************************************************/
	function getGraphDimensionY()
	{
		$dim=$this->getGraphDimension();
		return $dim[1];
	}

	/****************************************************************************//**
	 * @brief initialisation du graphe
	 *
	 * Initialisation des paramètres communs aux graphes concernant les usages des
	 * ressources en valeurs absolues
	 *
	 * @param 	$graph
	 * @return	$graph
	 *********************************************************************************/
	function makeNumGraphInitGraph($graph)
	{
		$graph->title->SetFont(FF_VERDANA,FS_NORMAL,10);
		$graph->SetScale( 'textlin');
		// Marges et position de la légende
		$graph->img->SetMargin(75,50,50,100);
		$graph->legend->Pos(0.02,0.05);
		$graph->xaxis->SetLabelAngle(45);
		$graph->xaxis->SetFont(FF_VERDANA,FS_NORMAL,8);
		$graph->xaxis->SetColor('black');
		$graph->xaxis->SetLabelMargin(10);
		$graph->yaxis->scale->SetGrace(10);
		// Titres et fontes
		$graph->xaxis->title->Set("Groupe");
		$graph->xaxis->SetTitleMargin(70);
		$graph->xaxis->title->SetFont(FF_FONT1,FS_BOLD);	
		$graph->yaxis->title->Set($this->getYaxisTitle());
		$graph->yaxis->SetTitleMargin(50);
		$graph->yaxis->title->SetFont(FF_FONT1,FS_BOLD);
		return $graph;
	}

	/****************************************************************************//**
	 * @brief initialisation des barres du graphe
	 *
	 * Initialisation des barres (communs aux graphes concernant les usages des
	 * ressources en valeurs absolues)
	 *
	 * @param 	$tabvalues,$tabcolors
	 * @return	$bplot
	 *********************************************************************************/
	function makeNumGraphInitPlot($tabvalues,$tabcolors)
	{
		$bplot = new BarPlot($tabvalues);
		// Spécification des couleurs
		$bplot->SetFillColor($tabcolors);
		$bplot->SetWidth(0.6);
		$bplot->SetWeight(1);
		// Affichage des valeurs
		$bplot->value->Show(true);
		$bplot->value->SetFormat('%d');
		return $bplot;
	}

	/****************************************************************************//**
	 * @brief Création du Line Plot
	 *
	 * Création des line plots (communs aux graphes concernant les usages des
	 * ressources en valeurs absolues)
	 *
	 * @param 	$tabvalues
	 * @return	$linplot
	 *********************************************************************************/
	function makeNumGraphInitLinePlot($tabvalues)
	{
		// Création du Line Plot
		$linplot= new LinePlot($tabvalues);
		$linplot->SetFillColor('lightblue@0.5');
		$linplot->SetColor('navy@0.7');
		$linplot->SetBarCenter();
		$linplot->mark->SetType(MARK_SQUARE);
		$linplot->mark->SetColor('blue@0.5');
		$linplot->mark->SetFillColor('lightblue');
		$linplot->mark->SetSize(6);

		switch ($this->graphvh->getGraphType())
		{
			case "numwalltimenbjobs" :
				$lintitle = "Nb Jobs";
				break;
			case "avgmemjobs" :
				$lintitle = "Mem max";
				break;
		}
		$linplot->setLegend($lintitle);
		return $linplot;
	}

	/****************************************************************************//**
	 * @brief Création des graphes NumWalltime et Cputime
	 *
	 * Création des graphes concernant les usages des ressources en valeurs absolues
	 * avec ou sans le temps disponible
	 * Pour créer le graphe les données nécessaire sont le nom des ressources,
	 * la valueur absolue par ressource et la couleur affectée à la ressource
	 *
	 * @param 	aucun
	 * @return	$img
	 *********************************************************************************/
	function makeNumtimeGraph()
	{
		// Definition des caratéristiques du graph
		// Dimension
		$graph = new Graph($this->getGraphDimensionX(),
		$this->getGraphDimensionY(),"auto");
		// Titres
		$graph->tabtitle->Set("Groupes applicatifs");
		$graph->title->Set($this->getGraphTitle());
		// Autres caractéristiques du graphe
		$graph = $this->makeNumGraphInitGraph($graph);
		
		// Traitement des données
		foreach ($this->datavh as $key => $value)
		{
			$enreg	     	=$this->datavh[$key];
			$tabgroup[]		= $enreg[0];
			$tabvalues1[]	= $enreg[1];
			if ($this->graphvh->getGraphType()=="numwalltimenbjobs" or $this->graphvh->getGraphType()=="avgmemjobs")
			{
				$tabvalues2[]	= $enreg[3];
			}
			if ($enreg[2]=="")
			{ //affectation d'une couleur par défaut au cas ou aucune couleur ne serait définie
				$tabcolors[] = "#000000"; //Couleur par Défaut
			}
			else
			{
				$tabcolors[] 	= $enreg[2];
			}
		}
		// Cas ou il n'y a pas de données
		if (array_sum($tabvalues1)==0)
		{
			$img = $this->createEmptyGraph();
		}
		else
		{
			// Titre de l'axe horizontal
			$graph->xaxis->SetTickLabels($tabgroup);
			// Ajout des barres au graphe
			$bplot = $this->makeNumGraphInitPlot($tabvalues1,$tabcolors);
			$graph->Add($bplot);
			// Ajout du line plot spécifique 
			if ($this->graphvh->getGraphType()=="numwalltimenbjobs" or $this->graphvh->getGraphType()=="avgmemjobs")
			{
				$linplot = $this->makeNumGraphInitLinePlot($tabvalues2);
				$graph->Add($linplot);
			}	
			// Création du graphe
			$img = $graph->Stroke($this->graphvh->getGraphfilename());
		}
		return $img;
	}


	/****************************************************************************//**
	 * @brief Création des graphes PercWalltime
	 *
	 * Création des graphes concernant les usages des ressources en pourcentage
	 * avec ou sans le temps disponible
	 * Pour créer le graphe les données nécessaire sont le nom des ressources,
	 * la valueur absolue par ressource et la couleur affectée à la ressource
	 *
	 * @param 	aucun
	 * @return	$img
	 *********************************************************************************/
	function makePercGraph()
	{
		// Definition des caratéristiques du graph
		// Dimension
		$graph = new PieGraph($this->getGraphDimensionX(),
		$this->getGraphDimensionY());
		// Titre
		$graph->title->Set($this->getGraphTitle());
		$graph->title->SetFont(FF_VERDANA,FS_NORMAL,10);
		$graph->SetScale( 'textlin');
		// Marges et position de la légende
		$graph->legend->Pos(0.02,0.15);

		// Traitement des données
		foreach ($this->datavh as $key => $value)
		{
			$enreg	     =$this->datavh[$key];
			$piegroups[] =$enreg[0];
			$pievalues[] =$enreg[1];
			if ($enreg[2]=="")
			{ //affectation d'une couleur par défaut au cas ou aucune couleur ne serait définie
				$piecolors[] = "#000000"; //Couleur par Défaut
			}
			else
			{
				$piecolors[] 	= $enreg[2];
			}
		}
		if (array_sum($pievalues)==0)
		{
			$img = $this->createEmptyGraph();
		}
		else
		{
			$graph->SetShadow();
			$graph->legend->SetReverse();
			$legend=$this->buildLegendPercWallTime($pievalues,$piegroups);
			$pie = new PiePlot($pievalues);
			$pie->setColor("#000000");
			$pie->value->Show(FALSE);
			$pie->SetCenter(0.35);
			$pie->SetSliceColors($piecolors);
			$pie->SetLegends($legend);
			$graph->Add($pie);
			// Création du graphe
			$img = $graph->Stroke($this->graphvh->getGraphfilename());
		}
		return $img;
	}
	 
	/****************************************************************************//**
	 * @brief Création des graphes NumGraph
	 *
	 *
	 * @param 	aucun
	 * @return	$img
	 *********************************************************************************/
	function makeNumGraph()
	{
		// Definition des caratéristiques du graph
		// Dimension
		$graph = new Graph($this->getGraphDimensionX(),
		$this->getGraphDimensionY(),"auto");
		// Titres
		$graph->tabtitle->Set("Groupes applicatifs");
		$graph->title->Set($this->getGraphTitle());
		// Autres caractéristiques du graphe
		$graph = $this->makeNumGraphInitGraph($graph);
		
		// Traitement des données
		foreach ($this->datavh as $key => $value)
		{
			$enreg	     	=$this->datavh[$key];
			$tabgroup[]		= $enreg[0];
			$tabvalues1[]	= $enreg[1];
			if ($enreg[2]=="")
			{ //affectation d'une couleur par défaut au cas ou aucune couleur ne serait définie
				$tabcolors[] = "#000000"; //Couleur par Défaut
			}
			else
			{
				$tabcolors[] 	= $enreg[2];
			}
		}
		// Cas ou il n'y a pas de données
		if (array_sum($tabvalues1)==0)
		{
			$img = $this->createEmptyGraph();
		}
		else
		{
			// Titre de l'axe horizontal
			$graph->xaxis->SetTickLabels($tabgroup);
			// Ajout des barres au graphe
			$bplot = $this->makeNumGraphInitPlot($tabvalues1,$tabcolors);
			$graph->Add($bplot);
			// Création du graphe
			$img = $graph->Stroke($this->graphvh->getGraphfilename());
		}
		return $img;
	}
	 
	/****************************************************************************//**
	 * @brief Création des légendes PercWalltime
	 *
	 * Création des légendes pour les graphes concernant les usages des ressources
	 * en pourcentage avec ou sans le temps disponible
	 *
	 * @param 	$data,$grouplist
	 * @return	$grouplist
	 *********************************************************************************/
	function buildLegendPercWallTime($data,$grouplist)
	{
		// Les valeurs sont affichées dans la légende des camemberts
		$datasum= array_sum($data);
		if ($datasum>0)
		// Alignement des noms et des valeurs
		$textlength=0;
		foreach ($grouplist as $group)
		{
			if (strlen($group)>$textlength) $textlength=strlen($group);
		}
		for  ($i=0;$i<count($grouplist);$i++)
		{
			$grstrlen=strlen($grouplist[$i]);
			for ($j=0;$j<$textlength-$grstrlen;$j++)
			{
				$grouplist[$i] .= " ";
			}
			$grouplist[$i] .= " (";
			if (100.*$data[$i]/$datasum<10)
			$grouplist[$i] .= " ";
			$grouplist[$i] .= number_format(100.*$data[$i]/$datasum,1,'.','')."%%)";
		}
		return $grouplist;
	}

	/****************************************************************************//**
	 * @brief Création du titre de l'axe Y
	 *
	 * Création du titre de l'axe Y
	 *
	 * @param 	
	 * @return	$yaxistitle
	 *********************************************************************************/
	function getYaxisTitle()
	{
		switch ($this->graphvh->getGraphType())
		{
			case "numcputime":
				$yaxistitle = "Occupation des ressources en Heures";
				break;
			case "numwalltime" :
				$yaxistitle = "Occupation des ressources en Heures";
				break;
			case "numwalltimewa" :
				$yaxistitle = "Occupation des ressources en Heures";
				break;
			case "daywalltime":
				$yaxistitle = "Occupation des ressources en Heures";
				break;
			case "percnbjobs" :
				$yaxistitle = "Nombre de jobs";
				break;
			case "numnbjobs" :
				$yaxistitle = "Nombre de jobs";
				break;
			case "daynbjobs" :
				$yaxistitle = "Nombre de jobs";
				break;
			case "numwalltimenbjobs":  
				$yaxistitle = "";
				break;
			case "avgtimes":     
				$yaxistitle = "";
				break;
		}
		return $yaxistitle;
	}

	/****************************************************************************//**
	 * @brief initialisation du graphe
	 *
	 * Initialisation des paramètres communs aux graphes concernant les usages des
	 * ressources par période (jour, semaine, mois)
	 *
	 * @param 	$graph
	 * @return	$graph
	 *********************************************************************************/
	function makeDayGraphInitGraph($graph)
	{
		$graph->title->SetFont(FF_VERDANA,FS_NORMAL,10);
		$graph->SetScale( 'textlin');
		// Margins and Legend position
		$graph->legend->Pos(0.02,0.05);
		$graph->img->SetMargin(75,200,50,80);
		// Ajouter une ombre au conteneur
		$graph->SetShadow();
		$graph->SetMarginColor('silver');
		$graph->xaxis->SetLabelAngle(45);
		// Titles and fonts for the X and Y axes
		$graph->xaxis->title->Set('Date');
		$graph->xaxis->title->SetFont(FF_FONT1,FS_BOLD);
		$graph->xaxis->SetTitleMargin(40);
		$graph->xaxis->SetFont(FF_VERDANA,FS_NORMAL,8);
		$graph->xaxis->title->SetColor('black');

		$graph->yaxis->title->Set($this->getYaxisTitle());
		$graph->yaxis->title->SetFont(FF_FONT1,FS_BOLD);
		$graph->yaxis->SetTitleMargin(50);
		$graph->yaxis->SetFont(FF_VERDANA,FS_NORMAL,8);
		$graph->yaxis->SetColor('black');

		return $graph;
	}

	/****************************************************************************//**
	 * @brief Retoune la légende des périodes
	 *
	 * Retoune la légende des périodes par période (jour, semaine, mois)
	 *
	 * @param 	$datesrc
	 * @return	$datedest
	 *********************************************************************************/
	function getDayDateLegend($datesrc)
	{
		switch ($this->graphvh->getOptionPeriode())
		{
			case "day":
				$datedest = $datesrc;
				break;

			case "week":
				$week_nb = substr($datesrc, -2);
				$datedest = "Sem ".$week_nb ." ". substr($datesrc,0,4);
				break;
				
			case "month":
				$month_nb = substr($datesrc, -2);
				$datedest = $month_nb ." - ". substr($datesrc,2,2);
				break;
		}
		return $datedest;
	}

	/****************************************************************************//**
	 * @brief Retoune les "Bar plot" pour le graph par période
	 *
	 * Retoune les "Bar plot" par période (jour, semaine, mois)
	 *
	 * @param 	$tabcompteur,$tabgroups,$tabvalues,$tabcolors
	 * @return	$GroupBarPlot
	 *********************************************************************************/
	function makeDayGraphInitBarPlot($tabcompteur,$tabgroups,$tabvalues,$tabcolors)
	{
		$nb_periods=count($tabcompteur);
		for ($i=0;$i<$nb_periods;$i++)
		{
			$linearray[$tabgroups[$i]][$tabcompteur[$i]] = $tabvalues[$i];
		}
		$i=0;
		foreach ($linearray as $key => $value)
		{
			$bplot = new BarPlot($linearray[$key]);
			$bplot->SetLegend($key);
			$bplot->SetShadow('black@0.4');
			$bplot->SetFillColor($tabcolors[$i]);
			//$bplot->SetShadow();
			//$bplot->SetWidth(0.5);
			//plot en 3D
			//$bplot->SetFillGradient(  '#440000',$tabcolors[$i],GRAD_LEFT_REFLECTION);
			$i++;
			$GroupBarPlot[] = $bplot;
		}
		return $GroupBarPlot;
	}
	
	/****************************************************************************//**
	 * @brief Retoune les "Line plot" pour le graph 
	 *
	 * Retoune le "Line plot"
	 *
	 * @param 	$tabvalues
	 * @return	$linplot
	 *********************************************************************************/
	function makeDayGraphInitLinePlot($tabvalues)
	{
		// Extraction des valeurs du tableau
		foreach ($tabvalues as $key => $value)
		{
			$enreg=$tabvalues[$key];
			$tabvalues1[] 	= $enreg[0];
		}
		// Création du Line Plot
		$linplot= new LinePlot($tabvalues1);
		//Couleur de remplissage
		$linplot->SetFillColor('lightred@0.8');
		// Couleur et épaisseur de la courbe
		$linplot->SetColor('red@0');
		$linplot->SetWeight(1);
		$linplot->SetBarCenter();
		// Légende
		$linplot->SetLegend("Full CPU capacity");
		// Affichage de point de valeur
		$linplot->mark->SetType(MARK_SQUARE);
		$linplot->mark->SetFillColor('lightred');
		$linplot->mark->SetSize(6);

		return $linplot;
	}
	
	/****************************************************************************//**
	 * @brief Retoune le graph par période
	 *
	 * Retoune le graph par période (jour, semaine, mois)
	 *
	 * @param 	none
	 * @return	$img
	 *********************************************************************************/
	function makeDayGraph()
	{
		// Definition des caratéristiques du graph
		// Dimension
		$graph = new Graph($this->getGraphDimensionX(),
		$this->getGraphDimensionY(),"auto");
		// Titre du graph
		$graph->title->Set($this->getGraphTitle());
		// Initialisation des variables du graph
		$graph = $this->makeDayGraphInitGraph($graph);

		// Traitement des données
		$tmpdate=NULL;
		$compteurdate=-1;
		if (!is_array($this->datavh[0]))
		{
			print "erreur de tableau";
		}
		foreach ($this->datavh[0] as $key => $value)
		{
			$enreg=$this->datavh[0][$key];
			if ($enreg[0]!=$tmpdate)
			{
				$compteurdate++;
				$tmpdate 	= $enreg[0];
				//$tabprocs[] 	= $enreg[4];
				$tabdates[] 	= $this->getDayDateLegend($enreg[0]);

			}
			$tabcompteur[] = $compteurdate;
			$tabgroups[] 	= $enreg[1];
			$tabvalues[] 	= $enreg[2];
			if ($enreg[3]=="")
			{ //affectation d'une couleur par défaut au cas ou aucune couleur ne serait définie
				$tabcolors[] = "#000000"; //Couleur par Défaut
			}
			else
			{
				$tabcolors[] 	= $enreg[3];
			}
		}
		if (array_sum($tabvalues)==0)
		{
			$img = $this->createEmptyGraph();
		}
		else
		{
			// Ajout des "Bar plot"
			$GroupBarPlot = $this->makeDayGraphInitBarPlot($tabcompteur,$tabgroups,$tabvalues,$tabcolors);
			// Caption for the horizontal axis
			$graph->xaxis->SetTickLabels($tabdates);
			$gbplot = new AccBarPlot($GroupBarPlot);
			$gbplot->SetWidth(0.6);
			
			if ($this->graphvh->getGraphType()=="daywalltime")
			{
				$linplot = $this->makeDayGraphInitLinePlot($this->datavh[1]);
				$graph->Add($linplot);
			}
			$graph->Add($gbplot);
			// Création du graph
			$img = $graph->Stroke($this->graphvh->getGraphfilename());
		}
		return $img;
	}

	/****************************************************************************//**
	 * @brief Graphe vide
	 *
	 * Création d'un graphe vide s'il n'y a pas de données
	 *
	 * @param 	aucun
	 * @return	$graph
	 *********************************************************************************/
	function createEmptyGraph()
	{
		$graph = new CanvasGraph(600,300);
		$gtext = new Text("NO DATA FOR GRAPH");
		$gtext->SetPos(0.05,0.5);
		$gtext->SetOrientation("h");
		$gtext->SetFont(FF_FONT1,FS_NORMAL);
		$gtext->SetBox("white","black",'gray');
		$gtext->SetColor("black");
		$graph->AddText($gtext);
		$graph->Stroke($this->graphvh->getGraphfilename());
	}

	/****************************************************************************//**
	 * @brief Test de bibliothèques jpgraph
	 *
	 * Test de création d'un graph pour vérifier les bibliothèque jpgraph
	 * Le graphe est basé sur un exemple de jpgraph
	 *
	 * @param 	aucun
	 * @return	$graph
	 *********************************************************************************/
	function testJpGraph()
	{
		$now = time();
		$data1y=array(-8,8,9,3,5,6);
		$data2y=array(18,2,1,7,5,4);

		// Création du graphe
		$graph = new Graph(500,400);
		$graph->SetScale("textlin");
		$graph->SetShadow();
		$graph->img->SetMargin(40,30,20,40);

		// Création des barres
		$b1plot = new BarPlot($data1y);
		$b1plot->SetFillColor("orange");
		$b1plot->value->Show();
		$b2plot = new BarPlot($data2y);
		$b2plot->SetFillColor("blue");
		$b2plot->value->Show();

		// Groupement des barres
		$gbplot = new AccBarPlot(array($b1plot,$b2plot));
		$graph->Add($gbplot);

		$graph->title->Set('Test Jpgraph - Date: '.date('Y-m-d H:i',$now));
		$graph->xaxis->title->Set("X-title");
		$graph->yaxis->title->Set("Y-title");
		$graph->title->SetFont(FF_FONT1,FS_BOLD);
		$graph->yaxis->title->SetFont(FF_FONT1,FS_BOLD);
		$graph->xaxis->title->SetFont(FF_FONT1,FS_BOLD);
		$graph->Stroke($this->graphvh->getGraphfilename());
	}

}

?>
