<?php
/****************************************************************************//**
 * @file	accgraphparam.inc.php
 * @brief	Classe de gestion des paramètres des graphes
 *
 * Les différents paramètres (interval de dates, type de graphe, ...) sont stockés
 * et gérés par cette classe.
 *
 * @author Eric
 * @version 0.5
 *********************************************************************************/

/****************************************************************************//**
 * @brief Classe accGraphparam
 *
 * Classe de gestion des paramètres des graphes
 *
 * @param	private $graph_type; 	-> Type de graphe choisi
 * @param	private $date_start; 	-> Date de début de sélection des données
 * @param	private $date_end;		-> Date de fin de sélection des données
 * @param	private $option_periode;-> Option de la période (cumul semaine/mois)
 * @param	private $jobs;			-> Type de jobs (locaux/externe/mpi/tous)
 * @param	private $surcharge;		-> Surcharge
 * @param	private $groups;		-> Liste de groupes (account/group) sélectionnés
 *
 *
 *********************************************************************************/

class accGraphparam {
	/****************************************************************************//**
	 * Déclaration des variables
	 *********************************************************************************/
	private $graph_type;
	private $date_start;
	private $date_end;
	private $option_periode;
	private $jobs;
	private $surcharge;
	private $procs;
	private $groups;
	private $graphfilename;

	/****************************************************************************//**
	 * Déclaration des méthodes
	 *********************************************************************************/

	/****************************************************************************//**
	 * @brief Constructeur
	 *
	 * Initialisation de l'objet
	 *
	 * @param 	$pgraph_type		-> Type de graphe choisi
	 * @param	$pdate_start		-> Date de début de sélection des données
	 * @param	$pdate_end			-> Date de fin de sélection des données
	 * @param	$poption_periode	-> Option de la période (cumul semaine/mois)
	 * @param	$pjobs				-> Type de jobs (locaux/externe/mpi/tous)
	 * @param	$psurcharge			-> Surcharge
	 * @param	$pgroups			-> Liste de groupes (account/group) sélectionnés
	 * @param	$pgroups			-> Type de procs pris en compte
	 * @param	$pgraphfilename		-> Chemin et Nom d'enregistrement du fichier du graphe
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	function __construct($pgraph_type, $pdate_start, $pdate_end, $poption_periode, $pjobs, $psurcharge, $pgroups,$pprocs,$pgraphfilename)
	{
		if (isset($pgraph_type))
		{
			if (isset($pdate_start))
			{
				if (isset($pdate_end))
				{
					if (isset($poption_periode))
					{
						if (isset($pjobs))
						{
							if (isset($psurcharge))
							{
								if (isset($pprocs)) 
									{
									if (isset($pgraphfilename))
									{
										// Acceptation des paramètres
										$this->graph_type=$pgraph_type;
										$this->date_start=$pdate_start;
										$this->date_end=$pdate_end;
										$this->option_periode=$poption_periode;
										$this->jobs=$pjobs;
										$this->surcharge=$psurcharge;
										$this->groups=$pgroups;
										$this->procs=$pprocs;
										$this->graphfilename=$pgraphfilename;
									//return CT_OK;
									}
									else
									throw new accException("CT_ERR_GRAPHFILENAME"); // Erreur groupes
								}
								else
								throw new accException("CT_ERR_PROCS"); // Erreur groupes
								//return CT_ERR_GROUPS;
							}
							else
							throw new accException("CT_ERR_SURCHARGE");  // Erreur surcharge
							//return CT_ERR_SURCHARGE;
						}
						else
						throw new accException("CT_ERR_JOBS"); // Erreur jobs
						//return CT_ERR_JOBS;
					}
					else
					throw new accException("CT_ERR_PERIODE");  //Erreur de période
					//return CT_ERR_PERIODE;
				}
				else
				throw new accException("CT_ERR_DATE_END");  // Erreur Date
				//return  CT_ERR_DATE_END;
			}
			else
			throw new accException("CT_ERR_DATE_START"); // Erreur Date
			//return CT_ERR_DATE_START;
		}
		else
		throw new accException("CT_ERR_GRAPH_TYPE"); // Erreur graph_type
		//return CT_ERR_GRAPH_TYPE;
	}

	/****************************************************************************//**
	 * @brief Destructeur
	 *
	 * Destruction de l'objet
	 *********************************************************************************/

	function __destruct()
	{
		$this->graph_type="";
		$this->date_start="";
		$this->date_end="";
		$this->option_periode="";
		$this->jobs="";
		$this->surcharge="";
		$this->groups="";
		$this->procs="";
		$this->pgraphfilename="";
	}

	/****************************************************************************//**
	 * @brief Affichage des paramètres
	 *
	 * Affichage des paramètres sélectionnés pour le graphe
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	function afficheParam()
	{
		print "<br>Type de graph : " .$this->graph_type;
		print "<br>Début : " .$this->date_start;
		print "<br>Fin : " .$this->date_end;
		print "<br>Option Periode : " .$this->option_periode;
		print "<br>Type de Job : " .$this->jobs;
		print "<br>Surcharge : " .$this->surcharge;
		print "<br>Groupes : "; print_r($this->groups);
		print "<br>Procs : "; print_r($this->procs);
		print "<br>Nom du fichier : " .$this->graphfilename;
	}

	/****************************************************************************//**
	 * @brief Type de graphe sélectionné
	 *
	 * Retourne le type de graphe sélectionné
	 *
	 * @param 	aucun
	 * @return	graph_type
	 *********************************************************************************/
	function getGraphType()
	{
		return $this->graph_type;
	}

	/****************************************************************************//**
	 * @brief Date de début
	 *
	 * Retourne la date de début de sélection des données
	 *
	 * @param 	aucun
	 * @return	date_start
	 *********************************************************************************/
	function getDateStart()
	{
		return $this->date_start;
	}

	/****************************************************************************//**
	 * @brief Date de fin
	 *
	 * Retourne la date de fin de sélection des données
	 *
	 * @param 	aucun
	 * @return	date_start
	 *********************************************************************************/
	function getDateEnd()
	{
		return $this->date_end;
	}

	/****************************************************************************//**
	 * @brief Option de période
	 *
	 * Retourne l'option de période (cumul par semaine/mois)
	 *
	 * @param 	aucun
	 * @return	date_start
	 *********************************************************************************/
	function getOptionPeriode()
	{
		return $this->option_periode;
	}

	/****************************************************************************//**
	 * @brief Type de jobs
	 *
	 * Retourne le type de jobs (Tous - Locaux/Externes - Série/MPI)
	 *
	 * @param 	aucun
	 * @return	jobs
	 *********************************************************************************/
	function getJobs()
	{
		return $this->jobs;
	}

	/****************************************************************************//**
	 * @brief Surcharge
	 *
	 * Retourne le mode de surcharge
	 *
	 * @param 	aucun
	 * @return	surcharge
	 *********************************************************************************/
	function getSurcharge()
	{
		return 	$this->surcharge;
	}

	/****************************************************************************//**
	 * @brief Liste des groupes de sélection
	 *
	 * Retourne la liste des groupes de sélection (account et group)
	 *
	 * @param 	aucun
	 * @return	groups
	 *********************************************************************************/
	function getGroups()
	{
		return $this->groups;
	}
	
	/****************************************************************************//**
	 * @brief Type de procs sélectionné
	 *
	 * Retourne le type de procs sélectionné
	 *
	 * @param 	aucun
	 * @return	procs
	 *********************************************************************************/
	function getProcs()
	{
		return $this->procs;
	}

	/****************************************************************************//**
	 * @brief Nom du fichier du graph
	 *
	 * Retourne le chemin et le nom du fichier du graphe
	 *
	 * @param 	aucun
	 * @return	groups
	 *********************************************************************************/
	function getGraphfilename()
	{
		return $this->graphfilename;
	}
	
	/****************************************************************************//**
	 * @brief interval des dates
	 *
	 * Retourne l'interval des dates de début et de fin au format sql
	 *
	 * @param 	aucun
	 * @return	$interval
	 *********************************************************************************/
	function getDateIntervalAsSqlQueryString()
	{
		//$interval = "BETWEEN '" .$this->date_start."' AND '" . $this->date_end." 23:59:59"."'";
		$interval = "BETWEEN '" .$this->date_start."' AND '" . $this->date_end."'";
		return $interval;
	}

	/****************************************************************************//**
	 * @brief date de début
	 *
	 * Retourne la date de début  au format sql
	 *
	 * @param 	aucun
	 * @return	$interval
	 *********************************************************************************/
	function getSqlDateStart()
	{
		$date = $this->date_start." 00:00:00" ;
		return $date;
	}
	
	/****************************************************************************//**
	 * @brief date de fin
	 *
	 * Retourne la date fin au format sql
	 *
	 * @param 	aucun
	 * @return	$interval
	 *********************************************************************************/
	function getSqlDateEnd()
	{
		$date = $this->date_end." 23:59:59";
		return $date;
	}

	/****************************************************************************//**
	 * @brief interval des semaines
	 *
	 * Retourne l'interval des semaines de début et de fin au format sql
	 *
	 * @param 	aucun
	 * @return	$interval
	 *********************************************************************************/
	function getWeekIntervalAsSqlQueryString()
	{
		$interval = "BETWEEN yearweek(date('" .$this->date_start."' )) AND yearweek(date('" .$this->date_end."'))";
		return $interval;
	}

	/****************************************************************************//**
	 * @brief interval des mois
	 *
	 * Retourne l'interval des mois de début et de fin au format sql
	 *
	 * @param 	aucun
	 * @return	$interval
	 *********************************************************************************/
	function getMonthIntervalAsSqlQueryString()
	{
		$interval = "BETWEEN concat(year(date('" .$this->date_start."' )),\"-M\", date_format(date('" .$this->date_start."'),'%m')) AND concat(year(date('" .$this->date_end."' )),\"-M\", date_format(date('" .$this->date_end."'),'%m'))";
		return $interval;
	}

}

?>
