<?php
/****************************************************************************//**
 * @file	accsql.inc.php
 * @brief	Classe de gestion des requètes sql
 *
 * @author Eric
 * @version 1.0
 *********************************************************************************/

/****************************************************************************//**
 * @brief Classe accMysql
 *
 * Classe de gestion des connexions et requètes sql
 *
 * @param	String : private $server_name; 	-> Nom du server Mysql
 * @param	String : private $db_name;		-> Nom de la base Mysql
 * @param	String : private $user_name;	-> Nom de l'utilisateur Mysql
 * @param	String : private $user_password;-> Password de l'utilisateur Mysql
 * @param	String : private $sql_link;		-> Connexion Mysql
 * @param	Array  : public $result;		-> Résultat de la requète
 * @param	Int    : public $rowcount; 		-> Nombre de ligne de résultat
 *
 *
 *********************************************************************************/

class accSql {
	/****************************************************************************//**
	 * Déclaration des variables
	 ********************************************************************************/
	private $server_name;
	private $db_name;
	private $user_name;
	private $user_password;
	private $sql_link;
	public  $result;
	public  $rowcount;
	/****************************************************************************//**
	 * Déclaration des méthodes
	 *********************************************************************************/

	/****************************************************************************//**
	 * @brief Constructeur
	 *
	 * Initialisation de l'objet
	 *
	 * @param $server_name;		-> Nom du server Mysql
	 * @param $db_name;			-> Nom de la base Mysql
	 * @param $user_name;		-> Nom de l'utilisateur Mysql
	 * @param $user_password;	-> Password de l'utilisateur Mysql
	 *
	 * @throws 	accException 	-> Description défini pour chaque exception
	 *********************************************************************************/
	function __construct($server_name, $db_name, $user_name, $user_password)
	{
		if (isset($server_name))
		{
			if (isset($user_name))
			{
				if (isset($user_password))
				{
					if (isset($db_name))
					{
						// Acceptation des paramètres
						$this->server_name=$server_name;
						$this->db_name=$db_name;
						$this->user_name=$user_name;
						$this->user_password=$user_password;
						$this->sql_link=FALSE;
						$this->result=array();
						$this->rowcount=0;
					}
					else
					throw new Accexception("CT_ERR_MYDB_DB"); // Erreur nom de la base
				}
				else
				throw new Accexception("CT_ERR_MYDB_PASS"); // Erreur password du user
			}
			else
			throw new Accexception("CT_ERR_MYDB_USER"); // Erreur nom du user
		}
		else
		throw new Accexception("CT_ERR_MYDB_HOST"); // Erreur nom du serveur
	}

	/****************************************************************************//**
	 * @brief Destructeur
	 *
	 * Destruction de l'objet
	 *********************************************************************************/
	function __destruct()
	{
		$this->server_name='';
		$this->db_name='';
		$this->user_name='';
		$this->user_password='';
		$this->sql_link=FALSE;
		$this->result=array();
		$this->rowcount=0;
	}

	/****************************************************************************//**
	 * @brief Affichage des paramètres
	 *
	 * Affichage des paramètres de connexion à la base Mysql
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	public function afficheParam()
	{
		echo "Nom du serveur : " .$this->server_name;
		echo "Nom de la base : " .$this->db_name;
		echo "Nom de l'utilisateur : " .$this->user_name;
		echo "Password : " .$this->user_password;
	}

	/****************************************************************************//**
	 * @brief Verification de la connexion sql
	 *
	 * Vérifie que la connexion sql est établie ou crée une connexion si elle est inexistante
	 *
	 * @param 	aucun
	 * @return	"CT_OK"
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	public function verifConnexion()
	{ // Vérification de l'éxistance d'une connexion
		if (!$this->sql_link)
		{ // Crée la connexion
			//$this->afficheParam();
			$this->sql_link= new mysqli($this->server_name, $this->user_name, $this->user_password,$this->db_name);
			if ($this->sql_link->connect_error)
			{
				throw new accException(mysqli_connect_error());
			}
		}
		return "CT_OK";
	}

	/****************************************************************************//**
	 * @brief Effectue une requète sql
	 *
	 * Excécute une requète sql ($req) et stocke le résultat (result)
	 *
	 * @param 	$req
	 * @return	aucun
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	public function requeterow($req)
	{
		if (is_string($req))
		{ // Connexion à la base de données
			if ($this->verifConnexion() == "CT_OK")
			{
				$res = $this->sql_link->query($req);
				if ($res->num_rows==0)
				{
					$this->rowcount=0;
					$this->result="";
				}
				else
				{
					$this->rowcount=0; // initialisation du compteur
					$this->result=""; // initialisation du tableau
					// Récuparation du résultat de la requète
					while ($row=$res->fetch_row())
					{
						$this->result[$this->rowcount]=$row;
						$this->rowcount+=1;
					}
				}
				return "CT_OK";
			}
			else
			throw new accException(" CT_ERR_MYDB_CONNECT");
		}
		else
		throw new accException("CT_NO_REQ"); //Pas de requète
	}

	/****************************************************************************//**
	 * @brief Effectue une requète sql
	 *
	 * Excécute une requète sql ($req) et retourne le nombre d'enregistrement
	 *
	 * @param 	$req
	 * @return	$res->num_rows
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	public function requeterownb($req)
	{
		if (is_string($req))
		{ // Connexion à la base de données
			if ($this->verifConnexion() == "CT_OK")
			{
				$res = $this->sql_link->query($req);
				return $res->num_rows;	
			}
			else
			throw new accException(" CT_ERR_MYDB_CONNECT");
		}
		else
		throw new accException("CT_NO_REQ"); //Pas de requète
	}

	/****************************************************************************//**
	 * @brief Effectue une requète sql
	 *
	 * Excécute une requète sql ($req) et stocke le résultat (result)
	 *
	 * @param 	$req
	 * @return	aucun
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	public function requetesql($req)
	{
		if (is_string($req))
		{ // Connexion à la base de données
			if ($this->verifConnexion() == "CT_OK")
			{
				$res=mysqli_query($this->sql_link,$req) ;//or die(mysql_error()); // Exécution de la requète
				return "CT_OK";
			}
			else
			throw new accException(" CT_ERR_MYDB_CONNECT");
		}
		else
		throw new accException("CT_NO_REQ"); //Pas de requète
	}

	/****************************************************************************//**
	 * @brief Effectue une requète sql multiple
	 *
	 * Excécute plusieurs requètes sql ($req) et stocke le résultat (result)
	 *
	 * @param 	$req
	 * @return	aucun
	 *
	 * @throws 	accException 		-> Description défini pour chaque exception
	 *********************************************************************************/
	public function multirequetesql($req)
	{
		if (is_string($req))
		{ // Connexion à la base de données
			if ($this->verifConnexion() == "CT_OK")
			{
				if ($this->sql_link->multi_query($req))
				{
					do {
						/* Stockage du premier résultat */
						if ($result = $this->sql_link->store_result())
						{

							$result->free();
						}
					} while ($this->sql_link->next_result());
				}
				return "CT_OK";
			}
			else
			throw new accException(" CT_ERR_MYDB_CONNECT");
		}
		else
		throw new accException("CT_NO_REQ"); //Pas de requète
	}

	/****************************************************************************//**
	 * @brief Affiche le résultat d'une requète sql
	 *
	 * Affiche l'intégralité du résultat du contenu de la variable result
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	public function showSqlResult()
	{
		$i=0;
		while ($i<$this->rowcount)
		{
			print_r($this->result[$i]);
			$i++;
		}
	}

	/****************************************************************************//**
	 * @brief Vérifie que la base de données contient les vues nécessaires
	 *
	 * Création des vues génériques à la base de données accounting
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	public function setSgbdAccounting()
	{
		// Création des vues génériques
		// Vue pour CPU disponible
		$reqnbjobs = "create VIEW `vvcpuavailableday` AS SELECT date(`date`) as cday ,\"0\" as sprocs FROM calendar where calendar.date not in (select date from calcul) union all SELECT date(`date`) as cday, `vprocs`*24/2 as sprocs FROM calcul";
		$this->multirequetesql($reqnbjobs);
		$reqnbjobs = "create VIEW `vcpuavailableday` AS SELECT date(`date`) as cday ,\"0\" as sprocs FROM calendar where calendar.date not in (select date from calcul) union all SELECT date(`date`) as cday, `procs`*24 as sprocs FROM calcul";
		$this->multirequetesql($reqnbjobs);
	}

	/****************************************************************************//**
	 * @brief Date Pbsmaui
	 *
	 * Retourne la première date dans la table Pbsmaui
	 *
	 * @param
	 * @return	date
	 *********************************************************************************/
	function getFirstDatePbsmaui()
	{
		$reqsql="SELECT min(date) FROM pbsmaui;";
		$this->requeterow($reqsql);
		$row =  $this->result[0][0];
		return $row;
	}

	/****************************************************************************//**
	 * @brief Cpu disponible
	 *
	 * Retourne le cpu disponible sur sur une période
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	CPU*24
	 *********************************************************************************/
	function getAvailableCpuHour($gvh)
	{
		$dateselect="date ".$gvh->getDateIntervalAsSqlQueryString();
	//	$proctype="procs";
		switch ($gvh->getProcs())
		{
			case "procs": // Cpu total par jour.
				$reqsql="SELECT SUM(procs) FROM calcul WHERE $dateselect ";
				break;
					
			case "vprocs": // Cpu total par semaine.
				$reqsql="SELECT SUM(vprocs/2) FROM calcul WHERE $dateselect ";
				break;
					
		}
		// Exécution de la requète.
		
		$this->requeterow($reqsql);
		$row =  $this->result[0];
		return $row[0]*24;
	}

	/****************************************************************************//**
	 * @brief Retourne le Cpu disponible par période
	 *
	 * Retourne le Cpu disponible par période sous la forme d'un tableau
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataCpuAvailable($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldateinterval= $gvh->getDateIntervalAsSqlQueryString();
		//$proctype="procs";
		switch ($gvh->getProcs())
		{
			case "procs": // Cpu total par jour.
				$procsview="vcpuavailableday";
				break;
					
			case "vprocs": // Cpu total par semaine.
				$procsview="vvcpuavailableday";
				break;
					
		}
		// Groupement choisi
		switch ($gvh->getOptionPeriode())
		{
			case "day": // Cpu total par jour.
				$reqsqlcpu= "select sprocs from ".$procsview ." where cday " .$sqldateinterval ;
				break;
					
			case "week": // Cpu total par semaine.
				$sqldateweek= $gvh->getWeekIntervalAsSqlQueryString();
				$reqsqlcpu= "select sum(sprocs) from ".$procsview ." where cday " .$sqldateinterval ." group by yearweek(date(cday))"  ;
				break;
					
			case "month": // Cpu total par jour	mois.
				$sqldatemonth= $gvh->getMonthIntervalAsSqlQueryString();
				$reqsqlcpu= "select sum(sprocs) from ".$procsview ." where cday " .$sqldateinterval ." group by concat(year(date(cday)),\"-M\", date_format(date(cday),'%m'))" ;
				break;
		}
		// Exécution de la requète.
		$this->requeterow($reqsqlcpu);
		return $this->result;
	}
}
?>