<?php
/****************************************************************************//**
 * @file	accsqliphc.inc.php
 * @brief	Classe de gestion des requètes sql spécifique à l'iphc
 *
 * @author Eric
 * @version 0.8.1
 *********************************************************************************/

/****************************************************************************//**
 * @brief Classe accSqliphc
 *
 * Classe de gestion des requètes sql spécifiques à l'IPHC
 *
 *
 *********************************************************************************/

class accSqliphc extends accSql {
	/****************************************************************************//**
	 * Déclaration des variables
	 ********************************************************************************/

	/****************************************************************************//**
	 * Déclaration des méthodes
	 *********************************************************************************/
	/****************************************************************************//**
	 * @brief Vérifie que la base de données contient les vues et tables spécifiques pour l'IPHC
	 *
	 * Si les tables et les vues spécifiques à l'IPHC n'existent pas elles sont créées
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	public function setSgbdIphc()
	{
		// Création des tables et vues spécifiques à l'IPHC
		// Vérifie que la table calendrier existe
		// Le calendrier est utilisé pour vérifier la cohérence des dates et qu'il n'y a pas de date manquante dans les données
		$reqcalendar = "Show tables like 'calendar'";

		if ($this->requeterownb($reqcalendar)==0)
		{
			//print "Création du calendrier";
			$reqcalendar = "drop view if exists v1,v2 ;";
			$reqcalendar .= "create table `calendar` (`date` DATE DEFAULT NULL ,PRIMARY KEY ( DATE )) ;";
			$reqcalendar .= "create view v1 as select 1 n union all select 1 union all select 1;";
			$reqcalendar .= "create view v2 as select 1 n from v1 a, v1 b union all select 1;";
			$reqcalendar .= "set @n = 0;";
			$reqcalendar .= "insert into calendar select '2000-1-1'+interval @n:=@n+1 day
		from v2 a, v2 b, v2 c, v2 d, v2 e, v2 limit 15000;";
			$reqcalendar .= "drop view if exists v1,v2 ;";
			$this->multirequetesql($reqcalendar);
		}
			
		// Vérifie que la table selector existe
		// La table selector permet de créer la liste des accounts et groupes utilisée pour la sélection lors des requètes (la synthese est connu comme Legend)
		// Une couleur est affectée à chaque enregistrement
		$reqselector = "Show tables like 'selector'";
		if ($this->requeterownb($reqselector)==0)
		{
			// Création de la table selector;
			$reqselector = "CREATE TABLE selector ( `sel_label` VARCHAR(16) CHARACTER SET utf8 COLLATE utf8_unicode_ci,
						 `sel_group` VARCHAR(16) CHARACTER SET utf8 COLLATE utf8_unicode_ci, 
						`sel_account` VARCHAR(16) CHARACTER SET utf8 COLLATE utf8_unicode_ci,
						`sel_color` VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_unicode_ci,
						`sel_legend` VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_unicode_ci, 
						UNIQUE (`sel_label`) ) CHARACTER SET utf8;";
			// Insertion des enregistrements account
			$reqselector .= "insert into selector (`sel_account`,`sel_label`,`sel_group`,`sel_color`,`sel_legend`)
						select distinct(account), Null, concat('a:', account), concat(\"#\",round(rand(600)*1000000)), 
						concat('Account ',account) from pbsmaui where account != \"[NONE]\" and account !=\"\" and 
						account not in (select account from selector) group by account;";
			// Insertion des enregistrements groupe
			$reqselector .= "insert into selector (`sel_group`,`sel_label`,`sel_account`,`sel_color`,`sel_legend`)
						select distinct(groupe), concat('g:', groupe),'[NONE]', concat(\"#\",round(rand(600)*1000000)), 
						concat('Group ',groupe) from pbsmaui where groupe != \"[NONE]\" and groupe !=\"\" and groupe not in 
						(select sel_group from selector) group by groupe;";
			$this->multirequetesql($reqselector);
		}

		// Création des vues
		// Vue pour le Walltime par jour
		$reqview = "create view vtimeday as SELECT calendar.date as gdate, IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend, sum(endtime - starttime ) as wtime
		from calendar, pbsmaui where pbsmaui.date = calendar.date and starttime < UNIX_TIMESTAMP(concat(calendar.date, ' 23:59:59')) and endtime >UNIX_TIMESTAMP(calendar.date) and calendar.date  group by pbsmaui.date, pbsmaui.groupe";
		$this->multirequetesql($reqview);

		// Vue pour le Walltime par semaine
		$reqview = "create view vtimeweek as SELECT yearweek(date(calendar.date)) as gdate, IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend, sum(endtime - starttime ) as wtime
		from calendar, pbsmaui where pbsmaui.date = calendar.date and starttime < UNIX_TIMESTAMP(concat(calendar.date, ' 23:59:59')) and endtime >UNIX_TIMESTAMP(calendar.date) and calendar.date  group by pbsmaui.date, pbsmaui.groupe";
		$this->multirequetesql($reqview);

		// Vue pour le Walltime par semaine
		$reqview = "create view vtimemonth as SELECT concat(year(date(calendar.date)),\"-M\", date_format(date(calendar.date),'%m')) as gdate, IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend, sum(endtime - starttime ) as wtime
		from calendar, pbsmaui where pbsmaui.date = calendar.date and starttime < UNIX_TIMESTAMP(concat(calendar.date, ' 23:59:59')) and endtime >UNIX_TIMESTAMP(calendar.date) and calendar.date  group by pbsmaui.date, pbsmaui.groupe";
		$this->multirequetesql($reqview);

		// Vue pour le nombre de jobs terminés par jour
		$reqnbjobs = "create view vnbjobsday as select calendar.date as gdate, IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend, count(pbsmaui.date) as totnbjobs
			from calendar, pbsmaui where pbsmaui.date = calendar.date and starttime < UNIX_TIMESTAMP(concat(calendar.date, ' 23:59:59')) and endtime >UNIX_TIMESTAMP(calendar.date) and calendar.date  group by pbsmaui.date, pbsmaui.groupe ";
		$this->multirequetesql($reqnbjobs);

		// Vue pour le nombre de jobs terminés par semaine
		$reqnbjobs = "create view vnbjobsweek as SELECT yearweek(date(pbsmaui.date)) as gdate, IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend, count(pbsmaui.date) as totnbjobs
			from calendar, pbsmaui where pbsmaui.date = calendar.date and starttime < UNIX_TIMESTAMP(concat(calendar.date, ' 23:59:59')) and endtime >UNIX_TIMESTAMP(calendar.date) and calendar.date group by pbsmaui.date, pbsmaui.groupe ";
		$this->multirequetesql($reqnbjobs);

		// Vue pour le nombre de jobs terminés par mois
		$reqnbjobs = "create view vnbjobsmonth as SELECT concat(year(date(pbsmaui.date)),\"-M\", date_format(date(pbsmaui.date),'%m')) as gdate, IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend, count(pbsmaui.date) as totnbjobs
			from calendar, pbsmaui where pbsmaui.date = calendar.date and starttime < UNIX_TIMESTAMP(concat(calendar.date, ' 23:59:59')) and endtime >UNIX_TIMESTAMP(calendar.date) and calendar.date group by pbsmaui.date, pbsmaui.groupe ";
		$this->multirequetesql($reqnbjobs);

		// Vue pour le nombre de jobs
		$reqnbjobs = "create VIEW `vnbjobs` AS select `pbsmaui`.`date` AS `date`,if((`pbsmaui`.`account` <> '[NONE]'),concat('Account ',`pbsmaui`.`account`),concat('Group ',`pbsmaui`.`groupe`)) AS `legend`,count(`pbsmaui`.`date`) AS `nbjobs` from `pbsmaui` group by `pbsmaui`.`date`,if((`pbsmaui`.`account` <> '[NONE]'),concat('Account ',`pbsmaui`.`account`),concat('Group ',`pbsmaui`.`groupe`))";
		$this->multirequetesql($reqnbjobs);

	}

	/****************************************************************************//**
	 * @brief Selection des groupes
	 *
	 * Requête pour l'affichage des groupes à sélectionner sur la page html
	 * Retourne le champs sel_legend de la table selector (spécifique à l'IPHC)
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	public function reqSelector()
	{
		$reqsql = "select sel_legend from selector;";
		$this->requeterow($reqsql);
	}

	/****************************************************************************//**
	 * @brief Legende des sel_legend (groupes/accounts) sélectionnés
	 *
	 * Retourne les sel_legend (groupes/accounts) sélectionnés sur l'interface Web
	 * Si aucun sel_legend n'est sélectionné, il sont tous sélectionnés par défaut
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$listlegend
	 *********************************************************************************/
	function getSelectionSel_Legend($gvh)
	{
		// Liste des groupes
		$list = $gvh->getGroups();
		// Gestion du cas où une partie des sel_legend est séletionnée
		if (!empty($list))
		{
			foreach($list as $selectvalue)
			{
				if ($i==0)
				{
					//$reqgroup= " and selector.sel_legend in (\"$selectvalue\" ";
					$reqgroup= " (\"$selectvalue\" ";
					$i++;
				}
				else
				{
					$reqgroup .= " , \"$selectvalue\" ";
				}
			}
			$reqgroup .= ") ";
		}

		// Si aucun sel_legend n'est sélectionné ils sont tous pris en compte (pratique pour la création automatique des graphes)
		else
		{
			$reqgroup= NULL;
		}

		return $reqgroup;
	}

	/****************************************************************************//**
	 * @brief Legende des sel_legend (groupes/accounts) non sélectionnés
	 *
	 * Retourne les sel_legend (groupes/accounts) non sélectionnés sur l'interface Web
	 * Si aucun sel_legend n'est sélectionné, il sont tous sélectionnés par défaut
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$listlegend
	 *********************************************************************************/
	function getNotSelectionSel_Legend($gvh)
	{
		// Récupération des sel_legend
		$list = $gvh->getGroups();
		// Gestion du cas où une partie des sel_legend est séletionnée
		if (!empty($list))
		{
			foreach($list as $selectvalue)
			{
				if ($i==0)
				{
					$reqgroup= " and selector.sel_legend not in (\"$selectvalue\" ";
					$i++;
				}
				else
				{
					$reqgroup .= " , \"$selectvalue\" ";
				}
			}
			$reqgroup .= ") ";
		}
		// Si aucun groupe n'est sélectionné ils sont tous pris en compte (util pour la création automatique des graphes)
		else
		{
			$reqgroup= " ";
		}

		return $reqgroup;
	}
	/****************************************************************************//**
	 * @brief Sélection de la requète SQL
	 *
	 * Sélection de la requète sql en fonction du type de graphe
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	aucun
	 *********************************************************************************/
	function getDataGraph($gvh)
	{
		switch ($gvh->getGraphType())
		{
			case "percwalltime": // Usage des ressources en pourcentage (% walltime)
				$data = $this->getDataWalltimeGraphIphc($gvh);
				break;

			case "percwalltimewa": // Usage des ressources sans le temps disponible en pourcentage (% walltime)
				$data = $this->getDataWalltimeWAGraphIphc($gvh);
				break;
					
			case "numcputime":  // Usage des ressources en valeurs absolues (cputime)
				$data = $this->getDataCputimeGraphIphc($gvh);
				break;

			case "numwalltime":  // Usage des ressources en valeurs absolues (walltime)
				$data = $this->getDataWalltimeGraphIphc($gvh);
				break;

			case "numwalltimewa":  // Usage des ressources sans le temps disponible en valeurs absolues (walltime)
				$data = $this->getDataWalltimeWAGraphIphc($gvh);
				break;

			case "daywalltime":  // Occupation des ressources par jour et par groupe
				$data[0] = $this->getDataPeriodGraphIphc($gvh);
				$data[1] = $this->getDataCpuAvailable($gvh);
				break;
				 
			case "percnbjobs":   // Nombre de jobs en pourcentage (%)
				$data = $this->getDataNumnbjobsGraphIphc($gvh);
				break;

			case "numnbjobs":    // Nombre de jobs en valeurs absolues
				$data = $this->getDataNumnbjobsGraphIphc($gvh);
				//$data[1] = $this->getDataNumnbjobsGraphIphc($gvh);
				break;

			case "daynbjobs":    // Nombre de jobs en valeurs absolues par jour
				$data[0] = $this->getDataDayGraphIphc($gvh);
				break;

			case "numwalltimenbjobs":     // Nombre de jobs terminés par jour et durées d'occupation
				$data = $this->getDataWalltimenbjobsGraphIphc($gvh);
				break;

			case "avgtimes":      // Durée moyenne des jobs terminés par groupe
				$data = $this->getDataWalltimenbjobsGraphIphc($gvh);
				break;
					
			case "avgmemjobs":     // Mémoire moyenne et maximale utilisée par les jobs terminés
				$data = $this->getDataMemGraphIphc($gvh);
				break;
					
			case "realmemjobs":     // Distribution de mémoire réelle utilisée par les jobs terminés
				$data = $this->getDataMemRealVirtualGraphIphc($gvh);
				break;
					
			case "virtualmemjobs":     // Distribution de mémoire virtuelle utilisée par les jobs terminés
				$data = $this->getDataMemRealVirtualGraphIphc($gvh);
				break;
					
			case "efficiencyjobs":     // Efficacité des jobs terminés
				$data = $this->getDataEfficiencyGraphIphc($gvh);
				break;
		}
		return($data);
	}

	/****************************************************************************//**
	 * @brief Renvoi le sens dans lequel les données doivent être trié
	 *
	 * Renvoi le sens ASC ou DESC de trie des données
	 * Ceci détermine le sens d'affichage des données dans les graphes
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	Sens du trie
	 *********************************************************************************/
	function getSortWalltime($gvh)
	{
		switch ($gvh->getGraphType())
		{
			case "percwalltime": 	// Usage des ressources en pourcentage (% walltime)
				$data = "ASC";
				break;

			case "percwalltimewa":	// Usage des ressources sans le temps disponible en pourcentage (% walltime)
				$data = "ASC";
				break;
					
			case "numwalltime":  	// Usage des ressources en valeurs absolues (walltime)
				$data = "DESC";
				break;

			case "numwalltimewa":  	// Usage des ressources sans le temps disponible en valeurs absolues (walltime)
				$data = "DESC";
				break;
				 
			case "percnbjobs":   	// Nombre de jobs en pourcentage (%)
				$data = "ASC";
				break;

			case "numnbjobs":    	// Nombre de jobs en valeurs absolues
				$data = "DESC";
				break;

			case "numwalltimenbjobs":     // Nombre de jobs terminés par jour et durées d'occupation
				$data = "DESC";
				break;

			case "avgtimes":      	// Durée moyenne des jobs terminés par groupe
				$data = "DESC";
				break;
		}
		return($data);
	}
	 
	/****************************************************************************//**
	 * @brief Usage des ressources
	 *
	 * Retourne l'usage des ressources sur une période sous la forme d'un tableau de 3 colonnes
	 * - La legende pour chaque ressource
	 * - Le temps utilisé pour chaque ressource
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataWalltimeGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne le temps cpu disponible
		$availablecpu = $this->getAvailableCpuHour($gvh);
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);
		// Retourne le sens de trie des enregistrements en fonction du type de graph
		$sortwalltime = $this->getSortWalltime($gvh);

		/* L'objectif est de sélectionner la somme de temps utilisé pour tous les enregistrements de pbsmaui entre 2 dates.
		 * Si le job commence avant ou termine après les dates sélectionnées, alors la partie de temps qui se trouve en dehors
		 * des dates sélectionnées n'est pas pris en compte.
		 *
		 * Le premier champs legend prend soit la valeur du champs account soit la valeur du champs groupe si account= [NONE]
		 * Le deuxième champs wtime est le temps entre starttime et endtime
		 * Si endtime > à la date de fin sélectionnée alors endtime prend la valeur de la date de fin sélectionnée
		 * Si starttime < à la date de début sélectionnée alors starttime prend la valeur de la date de début sélectionnée
		  
	 	Exemple de résultat obtenu par la requète générale:
	 	legend				wtime
	 	Account sbgbiomed	161901564279
	 	Account sbgcms		155428717377
	 	Account sbgyann		56890707629
	 	Group alice			22949107562857
	 	Group biomed		1285083139556
	 	Group cms			130192130460839
	 	*/

		// Définition de la requète générale
		$reqsqlpbsmaui= "select IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend,
			sum(IF(endtime<UNIX_TIMESTAMP('".$sqldateend."'),endtime,UNIX_TIMESTAMP('".$sqldateend."')) -
			IF(starttime>UNIX_TIMESTAMP('".$sqldatestart."'),starttime,UNIX_TIMESTAMP('".$sqldatestart."'))) as wtime
			from pbsmaui where starttime < UNIX_TIMESTAMP('".$sqldateend."') and 
			endtime >UNIX_TIMESTAMP('".$sqldatestart."') group by legend " ;

		// Gestion des groupes et account non présent sur la période sélectionnée
		// Ceci permet de mettre une valeur par défaut à 0 pour le temps de calcul pour les légendes (account, groupes) non trouvées sur une période
		// A voir : il serait intéressant de mettre cela en option
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as wtime from selector";
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;

		// Gestion de la sélection du choix des sel_legend (groupes/accounts)
		// Et ajout du temps restant disponible
		if (isset($listsel_legend)) // Cas de sélection des sel_legend (groupes/accounts)
		{
			$reqsql = $reqsql = "select legend as sel_legend,sum(wtime)/3600 as totwalltime,sel_color FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,sum(wtime)/3600 as totwalltime,\"#FFFF00\" FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				union 
				select \"Available\" as sel_legend, ".$availablecpu."-(sum(wtime)/3600) as totwalltime, \"#FFFFFF\" from (".$reqsqlpbsmaui.") as vuewalltime
				order by totwalltime ".$sortwalltime.", sel_legend ASC";
		}
		else
		{ // Tous les sel_legend (groupes/accounts) sont traités
			$reqsql = "SELECT legend as sel_legend,wtime/3600 as totwalltime,sel_color FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend group by legend
				union 
				select \"Available\" as sel_legend, ".$availablecpu."-(sum(wtime)/3600) as totwalltime, \"#FFFFFF\" from (".$reqsqlpbsmaui.") as vuewalltime 
				order by totwalltime ".$sortwalltime.", sel_legend ASC";
		}

		// Exécution de la requète
		$this->requeterow($reqsql);
		return $this->result;
	}

	/****************************************************************************//**
	 * @brief Usage des ressources sans le temps disponible restant
	 *
	 * Retourne l'usage des ressources, sans le temps disponible restant,
	 * sur une période et sous la forme d'un tableau de 3 colonnes
	 * - La legende pour chaque ressource
	 * - Le temps utilisé pour chaque ressource
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataWalltimeWAGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne le temps cpu disponible
		$availablecpu = $this->getAvailableCpuHour($gvh);
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);
		// Retourne le sens de trie des enregstrements en fonction du Walltime
		$sortwalltime = $this->getSortWalltime($gvh);

		/* L'objectif est de sélectionner la somme de temps utilisé pour tous les enregistrements de pbsmaui entre 2 dates.
		 * Si le job commence avant ou termine après les dates sélectionnées, alors la partie de temps qui se trouve en dehors
		 * des dates sélectionnées n'est pas pris en compte.
		 *
		 * Le premier champs legend prend soit la valeur du champs account soit la valeur du champs groupe si account= [NONE]
		 * Le deuxième champs wtime est le temps entre starttime et endtime
		 * Si endtime > à la date de fin sélectionnée alors endtime prend la valeur de la date de fin sélectionnée
		 * Si starttime < à la date de début sélectionnée alors starttime prend la valeur de la date de début sélectionnée
		  
	 	Exemple de résultat obtenu par la requète générale :
	 	legend				wtime
	 	Account sbgbiomed	161901564279
	 	Account sbgcms		155428717377
	 	Account sbgyann		56890707629
	 	Group alice			22949107562857
	 	Group biomed		1285083139556
	 	Group cms			130192130460839
	 	*/

		// Définition de la requète générale
		$reqsqlpbsmaui= "select IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend,
			sum(IF(endtime<UNIX_TIMESTAMP('".$sqldateend."'),endtime,UNIX_TIMESTAMP('".$sqldateend."')) -
			IF(starttime>UNIX_TIMESTAMP('".$sqldatestart."'),starttime,UNIX_TIMESTAMP('".$sqldatestart."'))) as wtime
			from pbsmaui where starttime < UNIX_TIMESTAMP('".$sqldateend."') and 
			endtime >UNIX_TIMESTAMP('".$sqldatestart."') group by legend " ;

		// Gestion des groupes et account non présent sur la période sélectionnée
		// Ceci permet de mettre une valeur par défaut à 0 pour le temps de calcul pour les légendes (account, groupes) non trouvées sur une période
		// A voir : il serait intéressant de mettre cela en option
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as wtime from selector";
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;

		// Gestion de la sélection du choix des sel_legend (groupes/accounts)
		if (isset($listsel_legend)) // Cas de sélection des sel_legend (groupes/accounts)
		{
			$reqsql = $reqsql = "select legend as sel_legend,sum(wtime)/3600 as totwalltime,sel_color FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,sum(wtime)/3600 as totwalltime,\"#FFFF00\" FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				order by totwalltime ".$sortwalltime.", sel_legend ASC";
		}
		else  // Si toutes des légendes ont été sélectionné dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
		{
			$reqsql = "SELECT legend as sel_legend,wtime/3600 as totwalltime,sel_color FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend group by legend
				order by totwalltime ".$sortwalltime.", sel_legend ASC";
		}

		// Exécution de la requète
		$this->requeterow($reqsql);
		return $this->result;
	}

	/****************************************************************************//**
	 * @brief Usage des ressources
	 *
	 * Retourne l'usage des ressources cpu sur une période sous la forme d'un tableau de 3 colonnes
	 * - La legende pour chaque ressource
	 * - Le cputime utilisé pour chaque ressource
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataCputimeGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne le temps cpu disponible
		$availablecpu = $this->getAvailableCpuHour($gvh);
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);

		/* L'objectif est de retourner le cputime utilisé pour tous les enregistrements de pbsmaui entre 2 dates.
		 * Si le job commence avant ou termine après les dates sélectionnées, alors la partie de cputime qui se trouve en dehors
		 * des dates sélectionnées n'est pas pris en compte.
		 *
		 * Le cputime n'étant connu que sous une valeur (champs cputime), la valeur est proratisée par rapport au temps endtime-starttime
		 * Le premier champs legend prend la valeur du champs account soit la valeur du champs groupe si account= [NONE]
		 * Le deuxième champs wtime est le temps entre starttime et endtime
		 * Si endtime > à la date de fin sélectionnée alors endtime prend la valeur de la date de fin sélectionnée
		 * Si starttime < à la date de début sélectionnée alors starttime prend la valeur de la date de début sélectionnée
		 */

		// Définition de la requête générale
		$reqsqlpbsmaui= "select IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend,
			sum(cputime *(IF(endtime<UNIX_TIMESTAMP('".$sqldateend."'),endtime,UNIX_TIMESTAMP('".$sqldateend."')) -
			IF(starttime>UNIX_TIMESTAMP('".$sqldatestart."'),starttime,UNIX_TIMESTAMP('".$sqldatestart."'))) / (endtime-starttime)) as ctime
			from pbsmaui where starttime < UNIX_TIMESTAMP('".$sqldateend."') and 
			endtime >UNIX_TIMESTAMP('".$sqldatestart."') group by legend " ;

		// Gestion des groupes et account non présent sur la période sélectionnée
		// Ceci permet de mettre une valeur par défaut à 0 pour le temps de calcul pour les légendes (account, groupes) non trouvées sur une période
		// A voir : il serait intéressant de mettre cela en option
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as ctime from selector";
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;

		// Gestion de la sélection de légendes dans le choix des groupes
		if (isset($listsel_legend)) // Cas de sélection des sel_legend (groupes/accounts)
		{
			$reqsql = $reqsql = "select legend as sel_legend,sum(ctime)/3600 as totcputime,sel_color FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,sum(ctime)/3600 as totcputime,\"#FFFF00\" FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				
				order by totcputime DESC, sel_legend ASC";
		}
		else // Si toutes les légendes sont sélectionnées dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
		{
			$reqsql = "SELECT legend as sel_legend,ctime/3600 as totcputime,sel_color FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend group by legend
				
				order by totcputime DESC, sel_legend ASC";		
		}
		
		/*	if (isset($listsel_legend)) // Cas de sélection des sel_legend (groupes/accounts)
		{
			$reqsql = $reqsql = "select legend as sel_legend,sum(ctime)/3600 as totcputime,sel_color FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,sum(ctime)/3600 as totcputime,\"#FFFF00\" FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				union 
				select \"Available\" as sel_legend, ".$availablecpu."-(sum(ctime)/3600) as totcputime, \"#FFFFFF\" from (".$reqsqlpbsmaui.") as vuecputime
				order by totcputime DESC, sel_legend ASC";
		}
		else // Si toutes les légendes sont sélectionnées dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
		{
			$reqsql = "SELECT legend as sel_legend,ctime/3600 as totcputime,sel_color FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend group by legend
				union 
				select \"Available\" as sel_legend, ".$availablecpu."-(sum(ctime)/3600) as totcputime, \"#FFFFFF\" from (".$reqsqlpbsmaui.") as vuecputime 
				order by totcputime DESC, sel_legend ASC";		
		} */

		// Exécution de la requète
		$this->requeterow($reqsql);
		return $this->result;
	}

	/****************************************************************************//**
	 * @brief Durée d'occupation des ressources par jour
	 *
	 * Retourne l'usage des ressources sur une période sous la forme d'un tableau de 4 colonnes
	 * - La date
	 * - La legende pour chaque ressource
	 * - Le (endtime - starttime) utilisé pour chaque ressource
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataPeriodGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldateinterval= $gvh->getDateIntervalAsSqlQueryString();
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);

		// Gestion du groupement par période (jour, semaine, mois)
		switch ($gvh->getOptionPeriode())
		{
			case "day": // Affichage par jour
				if (isset($listsel_legend)) // Cas où certains sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "select * from vtimeday where gdate " .$sqldateinterval ." and vtimeday.legend  in ".$listsel_legend  ;
					$reqsqllegendnontrouve=" union select calendar.date as gdate, sel_legend as legend, 0 as wtime from calendar, selector where calendar.date " .$sqldateinterval . "  and selector.sel_legend  in ".$listsel_legend ;
				}
				else  // Cas où tous les sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "select * from vtimeday where gdate " .$sqldateinterval  ;
					$reqsqllegendnontrouve=" union select calendar.date as gdate, sel_legend as legend, 0 as wtime from calendar, selector where calendar.date " .$sqldateinterval . "";
				}
				break;
					
			case "week": // Affichage par semaine
				$sqldateweek= $gvh->getWeekIntervalAsSqlQueryString();
				if (isset($listsel_legend)) // Cas où certains sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "select * from vtimeweek where gdate " .$sqldateweek ." and vtimeweek.legend  in ".$listsel_legend  ;
					$reqsqllegendnontrouve=" union select yearweek(date(calendar.date)) as gdate, sel_legend as legend, 0 as wtime from calendar, selector where calendar.date " .$sqldateinterval . " and selector.sel_legend  in ".$listsel_legend ;
				}
				else  // Cas où tous les sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "select * from vtimeweek where gdate " .$sqldateweek ;
					$reqsqllegendnontrouve=" union select yearweek(date(calendar.date)) as gdate, sel_legend as legend, 0 as wtime from calendar, selector where calendar.date " .$sqldateinterval . "";
				}
				break;
					
			case "month": // Affichage par mois
				$sqldatemonth= $gvh->getMonthIntervalAsSqlQueryString();
				if (isset($listsel_legend)) // Cas où certains sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "select * from vtimemonth where gdate " .$sqldatemonth ." and vtimemonth.legend  in ".$listsel_legend  ;
					$reqsqllegendnontrouve=" union select concat(year(date(calendar.date)),\"-M\", date_format(date(calendar.date),'%m')) as gdate, sel_legend as legend, 0 as wtime from calendar, selector where calendar.date " .$sqldateinterval . " and selector.sel_legend  in ".$listsel_legend ;
				}
				else  // Cas où tous les sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "select * from vtimemonth where gdate " .$sqldatemonth  ;
					$reqsqllegendnontrouve=" union select concat(year(date(calendar.date)),\"-M\", date_format(date(calendar.date),'%m')) as gdate, sel_legend as legend, 0 as wtime from calendar, selector where calendar.date " .$sqldateinterval . "" ;
				}
				break;
		}
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;
		// Groupement des enregistrements
		$reqsql = "select gdate, legend as sel_legend,sum(wtime)/3600 as totwalltime, selector.sel_color FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend
	group by gdate,legend order by gdate ASC, totwalltime DESC, sel_legend ASC";
		// Exécution de la requête
		$this->requeterow($reqsql);
		return $this->result;
	}

	/****************************************************************************//**
	 * @brief Nombre de jobs terminés par groupe
	 *
	 * Retourne le nombre de jobs terminés par groupe sur une période sous la forme d'un tableau de 3 colonnes
	 * - La legende pour chaque ressource
	 * - Le nb de jobs pour chaque ressource
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataNumnbjobsGraphIphc($gvh)
	{
		// Renvoi les dates de sélection
		$sqldateinterval= $gvh->getDateIntervalAsSqlQueryString();
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);
		// Retourne le sens de trie des enregistrements en fonction du type de graph
		$sortwalltime = $this->getSortWalltime($gvh);

		// Requête générale
		$reqsqlvnbjobs="SELECT  legend, sum(nbjobs) as totnbjobs, sel_color FROM vnbjobs,selector  where date ".$sqldateinterval." and vnbjobs.legend=selector.sel_legend group by legend " ;

		// Gestion des sel_legend (groupes/accounts) qui n'ont pas d'enregistrements pour la période donnée
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as totnbjobs,sel_color from selector where selector.sel_legend not in (select legend FROM `vnbjobs`  where date ".$sqldateinterval.") ";
		// Définition de l'odre de trie en fonction du graphe choisi
		$reqorder=" order by totnbjobs ".$sortwalltime."";
		// Union des requêtes
		$reqsqlnbjobs= $reqsqlvnbjobs . $reqsqllegendnontrouve . $reqorder ;


		if (isset($listsel_legend)) // Cas où certains sel_legend (groupes/accounts) sont sélectionnés
		{
			$reqsql = "select legend as sel_legend, totnbjobs ,sel_color FROM (".$reqsqlnbjobs.") as vuewalltime where vuewalltime.legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend, sum(totnbjobs) as totnbjobs ,\"#FFFF00\" FROM (".$reqsqlnbjobs.") as vuewalltime where vuewalltime.legend not in ".$listsel_legend." 
				order by totnbjobs ".$sortwalltime.", sel_legend ASC";
		}
		else // Cas où tous les sel_legend (groupes/accounts) sont sélectionnés
		{
			$reqsql = $reqsqlnbjobs;
		}
		// Exécution de la requête
		$this->requeterow($reqsql);
		return $this->result;
	}

	/****************************************************************************//**
	 * @brief Nombre de jobs terminés par groupe et par jour
	 *
	 * Retourne le nombre de jobs terminés par groupe sur une période sous la forme d'un tableau de 4 colonnes
	 * - La Date du jour
	 * - La legende pour chaque ressource
	 * - Le nb de jobs pour chaque ressource
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataDayGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldateinterval= $gvh->getDateIntervalAsSqlQueryString();
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);

		/*	Exemple de retour d'enregistrements
		 2010-09-01 	Account cmspriorityu 	0 		#91444
		 2010-09-01 	Account mpi 			0 		#CC0066
		 2010-09-01 	Account sbg 			0 		#CCCCCC
		 2010-09-01 	Account sbgbiomed 		137 	#130660
		 2010-09-01 	Account sbgcms 			362 	#55FF00
		 2010-09-01 	Account sbggeneric 		1	 	#893694
		 2010-09-01 	Account sbgimabio 		0 		#267660
		 2010-09-01 	Account sbgnemo 		0 		#424537
		 2010-09-01 	Account sbgramses 		0 		#770620
		 */

	 switch ($gvh->getOptionPeriode())
	 {
	 	case "day": // Affichage par jour
	 		if (isset($listsel_legend)) //Cas où certains sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "SELECT  * FROM vnbjobsday where gdate ".$sqldateinterval ." and vnbjobsday.legend  in ".$listsel_legend  ;
					$reqsqllegendnontrouve=" union select calendar.date as gdate, sel_legend as legend, 0 as totnbjobs from calendar, selector where calendar.date " .$sqldateinterval . " and selector.sel_legend  in ".$listsel_legend ;
				}
				else  // Cas où tous les sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui = "SELECT  * FROM vnbjobsday where gdate ".$sqldateinterval ;
					$reqsqllegendnontrouve=" union select calendar.date as gdate, sel_legend as legend, 0 as totnbjobs from calendar, selector where calendar.date " .$sqldateinterval . "";
				}
				break;
					
	 	case "week": // Affichage par semaine
	 		$sqldateweek= $gvh->getWeekIntervalAsSqlQueryString();
	 		if (isset($listsel_legend)) // Cas où certains sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "SELECT  * FROM vnbjobsweek where gdate ".$sqldateweek ." and vnbjobsweek.legend  in ".$listsel_legend  ;
					$reqsqllegendnontrouve=" union select  yearweek(date(calendar.date)) as gdate, sel_legend as legend, 0 as totnbjobs from calendar, selector where calendar.date " .$sqldateinterval . " and selector.sel_legend  in ".$listsel_legend  ;
				}
				else  // Cas où tous les sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui = "SELECT  * FROM vnbjobsweek where gdate ".$sqldateweek ;
					$reqsqllegendnontrouve=" union select  yearweek(date(calendar.date)) as gdate, sel_legend as legend, 0 as totnbjobs from calendar, selector where calendar.date " .$sqldateinterval . "";
				}
				break;
					
	 	case "month": // Affichage par mois
	 		$sqldatemonth= $gvh->getMonthIntervalAsSqlQueryString();
	 		if (isset($listsel_legend)) // Cas où certains sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui= "SELECT  * FROM vnbjobsmonth where gdate ".$sqldatemonth ." and vnbjobsmonth.legend  in ".$listsel_legend  ;
					$reqsqllegendnontrouve=" union select concat(year(date(date)),\"-M\", date_format(date(date),'%m')) as gdate, sel_legend as legend, 0 as totnbjobs from calendar, selector where calendar.date " .$sqldatemonth . " and selector.sel_legend  in ".$listsel_legend  ;
				}
				else  // Cas où tous les sel_legend (groupes/accounts) sont sélectionnés
				{
					$reqsqlpbsmaui = "SELECT  * FROM vnbjobsmonth where gdate ".$sqldatemonth ;
					$reqsqllegendnontrouve=" union select concat(year(date(date)),\"-M\", date_format(date(date),'%m')) as gdate, sel_legend as legend, 0 as totnbjobs from calendar, selector where calendar.date " .$sqldateinterval . "";
				}
				break;
	 }
	 // Union des requêtes
	 $reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;
	 // Groupement des enregistrements
	 $reqsql = "select gdate, legend as sel_legend,sum(totnbjobs) as totjobs, selector.sel_color FROM (".$reqsqlpbsmaui.") as vuenbjobs,selector where vuenbjobs.legend=selector.sel_legend
		group by gdate,legend order by gdate ASC, totjobs DESC, sel_legend ASC";
	 // Exécution de la requête
	 $this->requeterow($reqsql);
	 return $this->result;
	}

	/****************************************************************************//**
	 * @brief Usage des ressources (numWalltimenbjobs, avgtimes)
	 *
	 * Retourne l'usage des ressources sur une période sous la forme d'un tableau de 3 ou 4 colonnes (selon le type de graphe)
	 * - La legende pour chaque ressource
	 * - Le temps utilisé pour chaque ressource
	 * - La couleur pour chaque ressource
	 * - Le nombre de jobs
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataWalltimenbjobsGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne le temps cpu disponible
		$availablecpu = $this->getAvailableCpuHour($gvh);
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);
		// Retourne le sens de trie des enregstrements en fonction du Walltime
		$sortwalltime = $this->getSortWalltime($gvh);

		// Définition de la requête générale
		$reqsqlpbsmaui= "select IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend,
			sum(IF(endtime<UNIX_TIMESTAMP('".$sqldateend."'),endtime,UNIX_TIMESTAMP('".$sqldateend."')) -
			IF(starttime>UNIX_TIMESTAMP('".$sqldatestart."'),starttime,UNIX_TIMESTAMP('".$sqldatestart."'))) as wtime, count(pbsmaui.date) as totnbjobs
			from pbsmaui where starttime < UNIX_TIMESTAMP('".$sqldateend."') and 
			endtime > UNIX_TIMESTAMP('".$sqldatestart."') group by legend " ;

		// Gestion des groupes et account non présent sur la période sélectionnée
		// Ceci permet de mettre une valeur par défaut à 0 pour le temps de calcul pour les légendes (account, groupes) non trouvées sur une période
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as wtime, 0 as totnbjobs from selector";
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;
		// Création de la requête en fonction du type de graphe
		switch ($gvh->getGraphType())
		{
			case "numwalltimenbjobs":     // Nombre de jobs terminés par jour et durées d'occupation
				if (isset($listsel_legend)) // Si des sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes
				{
					$reqsql = $reqsql = "select legend as sel_legend,sum(wtime)/3600 as totwalltime,sel_color,sum(totnbjobs) as totjobs FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,sum(wtime)/3600 as totwalltime,\"#FFFF00\",sum(totnbjobs) as totjobs  FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				order by totwalltime ".$sortwalltime.", sel_legend ASC";
				}
				else  // Si tous les sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
				{
					$reqsql = "SELECT legend as sel_legend,wtime/3600 as totwalltime,sel_color,sum(totnbjobs) as totjobs  FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend group by legend
				order by totwalltime ".$sortwalltime.", sel_legend ASC";
				}
				break;

			case "avgtimes":      // Durée moyenne des jobs terminés par groupe
				if (isset($listsel_legend)) // Si des sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes
				{
					$reqsql = $reqsql = "select legend as sel_legend,wtime/3600/sum(totnbjobs) as totmoy,sel_color FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,wtime/3600/sum(totnbjobs) as totmoy,\"#FFFF00\"  FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				order by totmoy ".$sortwalltime.", sel_legend ASC";
				}
				else  // Si tous les sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
				{
					$reqsql = "SELECT legend as sel_legend,wtime/3600/sum(totnbjobs) as totmoy ,sel_color  FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend group by legend
				order by totmoy ".$sortwalltime.", sel_legend ASC";
				}
				break;
		}
		// Exécution de la requète
		$this->requeterow($reqsql);
		return $this->result;
	}

	/****************************************************************************//**
	 * @brief Mémoire moyenne et maximale utilisée par les jobs terminés
	 *
	 * Retourne l'usage des ressources sur une période sous la forme d'un tableau de 4 colonnes
	 * - La legende pour chaque ressource
	 * - La mémoire moyenne utilisée
	 * - La couleur pour chaque ressource
	 * - La mémoire maximum utilisée
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataMemGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne le temps cpu disponible
		$availablecpu = $this->getAvailableCpuHour($gvh);
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);
		// Retourne le sens de trie des enregstrements en fonction du Walltime
		$sortwalltime = $this->getSortWalltime($gvh);

		// Définition de la requête générale
		$reqsqlpbsmaui= "select IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend,
			sum(mem+vmem)/(count(pbsmaui.date)) as memmoy, max(mem+vmem) as memmax
			from pbsmaui where starttime < UNIX_TIMESTAMP('".$sqldateend."') and 
			endtime > UNIX_TIMESTAMP('".$sqldatestart."') group by legend " ;

		// Gestion des groupes et account non présent sur la période sélectionnée
		// Ceci permet de mettre une valeur par défaut à 0 pour le temps de calcul pour les légendes (account, groupes) non trouvées sur une période
		// A voir : il serait intéressant de mettre cela en option
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as memmoy, 0 as memmax from selector";
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;

		if (isset($listsel_legend)) // Si des sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes
		{
			$reqsql = $reqsql = "select legend as sel_legend, memmoy/1024 as tmemmoy,  sel_color, memmax/1024 as tmemmax  FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,sum(memmoy)/1024 as tmemmoy,\"#FFFF00\",sum(memmax)/1024 as tmemmax   FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				order by tmemmoy DESC, sel_legend ASC";
		}
		else  // Si tous les sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
		{
			$reqsql = "SELECT legend as sel_legend, memmoy/1024 as tmemmoy,  sel_color, memmax/1024 as tmemmax  FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend group by legend
				order by memmoy DESC, sel_legend ASC";
		}

		// Exécution de la requète
		$this->requeterow($reqsql);
		return $this->result;
	}


	/****************************************************************************//**
	 * @brief Mémoire réelle ou virtuelle utilisée par les jobs terminés
	 *
	 * Retourne l'usage des ressources sur une période sous la forme d'un tableau de 3 colonnes
	 * - La legende pour chaque ressource
	 * - La mémoire utilisée pour chaque ressource (réelle ou virtuelle)
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataMemRealVirtualGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne le temps cpu disponible
		$availablecpu = $this->getAvailableCpuHour($gvh);
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);
		// Retourne le sens de trie des enregstrements en fonction du Walltime
		$sortwalltime = $this->getSortWalltime($gvh);

		// Choix du type de mémoire à gérer en fonction du type de graphe choisi
		switch ($gvh->getGraphType())
		{
			case "realmemjobs":     // Distribution de mémoire réelle utilisée par les jobs terminés
				$memtype = "mem";
				break;
					
			case "virtualmemjobs":     // Distribution de mémoire virtuelle utilisée par les jobs terminés
				$memtype = "vmem";
				break;
		}
		// Définition de la requête générale
		$reqsqlpbsmaui= "select IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend,
			sum(".$memtype.")/(count(pbsmaui.date)) as memmoy
			from pbsmaui where starttime < UNIX_TIMESTAMP('".$sqldateend."') and 
			endtime > UNIX_TIMESTAMP('".$sqldatestart."') group by legend " ;

		// Gestion des groupes et account non présent sur la période sélectionnée
		// Ceci permet de mettre une valeur par défaut à 0 pour le temps de calcul pour les légendes (account, groupes) non trouvées sur une période
		// A voir : il serait intéressant de mettre cela en option
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as memmoy from selector";
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;

		if (isset($listsel_legend)) // Si des sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes
		{
			$reqsql = $reqsql = "select legend as sel_legend, memmoy/1024 as tmemmoy,  sel_color FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				union 
				select \"Autres\" as sel_legend,sum(memmoy)/1024 as tmemmoy,\"#FFFF00\" FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend and selector.sel_legend not in ".$listsel_legend." 
				order by tmemmoy DESC, sel_legend ASC";
		}
		else  // Si tous les sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
		{
			$reqsql = "SELECT legend as sel_legend, memmoy/1024 as tmemmoy,  sel_color  FROM (".$reqsqlpbsmaui.") as vuewalltime,selector where vuewalltime.legend=selector.sel_legend group by legend
				order by memmoy DESC, sel_legend ASC";
		}

		// Exécution de la requète
		$this->requeterow($reqsql);
		return $this->result;
	}

	/****************************************************************************//**
	 * @brief Efficacité des jobs
	 *
	 * Retourne l'efficacité des jobs sur une période sous la forme d'un tableau de 3 colonnes
	 * - La legende pour chaque ressource
	 * - L'efficatité des jobs (Walltime / Cputime)
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function  getDataEfficiencyGraphIphc($gvh)
	{
		// Renvoi les dates de sélection.
		$sqldatestart = $gvh->getSqlDateStart();
		$sqldateend = $gvh->getSqlDateEnd();
		// Retourne le temps cpu disponible
		$availablecpu = $this->getAvailableCpuHour($gvh);
		// Retourne la liste de sélection
		$listsel_legend = $this->getSelectionSel_Legend($gvh);

		// Requête générale
	$reqsqlpbsmaui= "select IF (pbsmaui.account!='[NONE]',concat('Account ', pbsmaui.account),concat('Group ',pbsmaui.groupe)) as legend,
					(sum(cputime) / sum(walltime))*100 as ctime
					from pbsmaui where starttime < UNIX_TIMESTAMP('".$sqldateend."') and 
					endtime >UNIX_TIMESTAMP('".$sqldatestart."') group by legend " ;
		
		// Gestion des groupes et account non présent sur la période sélectionnée
		// Ceci permet de mettre une valeur par défaut à 0 pour le temps de calcul pour les légendes (account, groupes) non trouvées sur une période
		// A voir : il serait intéressant de mettre cela en option
		$reqsqllegendnontrouve=" union select sel_legend as legend, 0 as ctime from selector";
		// Union des requêtes
		$reqsqlpbsmaui=$reqsqlpbsmaui . $reqsqllegendnontrouve;

		// Gestion de la sélection de sel_legend (groupes/accounts) dans le choix des groupes
		if (isset($listsel_legend)) // Si des sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes
		{
			$reqsql = $reqsql = "select legend as sel_legend,ctime as totcputime,sel_color FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend and selector.sel_legend in ".$listsel_legend." group by legend
				order by totcputime DESC, sel_legend ASC";
		}
		else // Si tous les sel_legend (groupes/accounts) ont été sélectionné dans le choix des groupes (par défaut si aucune légende dan le choix des groupes n'est sélectionnée, tous les légendes sont alors sélectionntées)
		{

			$reqsql = "SELECT legend as sel_legend,ctime as  totcputime,sel_color FROM (".$reqsqlpbsmaui.") as vuecputime,selector where vuecputime.legend=selector.sel_legend group by legend
				order by totcputime DESC, sel_legend ASC";			
		}
		// Exécution de la requète
		$this->requeterow($reqsql);
		return $this->result;
	}
}
?>
