<?php
/****************************************************************************//**
 * @file	accsqllapp.inc.php
 * @brief	Classe de gestion des requètes sql spécifique au lapp
 *
 * @author Eric
 * @version 0.2
 *********************************************************************************/

/****************************************************************************//**
 * @brief Classe accSqllapp
 *
 * Classe de gestion des connexions et requètes sql du Lapp
 *
 *
 *
 *********************************************************************************/

class accSqllapp extends accSql {
	/****************************************************************************//**
	 * Déclaration des variables
	 ********************************************************************************/

	/****************************************************************************//**
	 * Déclaration des méthodes
	 *********************************************************************************/
	/****************************************************************************//**
	 * @brief Sélection de la requète SQL
	 *
	 * Sélection de la requète sql en fonction du type de graphe
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	aucun
	 *********************************************************************************/
	function getDataGraph($gvh)
	{

		switch ($gvh->getGraphType())
		{
			case "percwalltime": // Usage des ressources en pourcentage (% walltime)
				$data = $this->getDataWalltimeGraphLapp($gvh);
				break;

			case "percwalltimewa": // Usage des ressources sans le temps disponible en pourcentage (% walltime)
				$data = $this->getDataWalltimeWAGraphLapp($gvh);
				break;

			case "numwalltime":  // Usage des ressources en valeurs absolues (walltime)
				$data = $this->getDataWalltimeGraphLapp($gvh);
				break;

			case "numwalltimewa":  // Usage des ressources sans le temps disponible en valeurs absolues (walltime)
				$data = $this->getDataWalltimeWAGraphLapp($gvh);
				break;
		}
		return($data);
	}

	/****************************************************************************//**
	 * @brief Usage des ressources
	 *
	 * Retourne l'usage des ressources sur une période sous la forme d'un tableau de 3 colonnes
	 * - La legende pour chaque ressource
	 * - Le temps utilisé pour chaque ressource
	 * - La couleur pour chaque ressource
	 *
	 * @param 	$gvh -> paramètres du graphe
	 * @return	$this->result
	 *********************************************************************************/
	function getDataWalltimeGraphLapp($gvh)
	{

		$sqldate= $gvh->getDateIntervalAsSqlQueryString();
		$availablecpu = $this->getAvailableCpuHour($gvh);

		$reqsql = "SELECT sel_legend,sum(walltime)/3600 as totwalltime,sel_color FROM pbsmaui,selector  where pbsmaui.date " .$sqldate. " and pbsmaui.groupe=selector.sel_group and selector.sel_account=\"[NONE]\" group by sel_legend
						Union
						SELECT sel_legend,sum(walltime)/3600 as totwalltime,sel_color FROM pbsmaui,selector  where pbsmaui.date " .$sqldate. " and pbsmaui.account=selector.sel_account and selector.sel_account!=\"[NONE]\" group by sel_legend
						union 
						select \"Available\", ".$availablecpu."-(sum(walltime)/3600) as totwalltime, \"#FFFFFF\" from pbsmaui where pbsmaui.date  " .$sqldate. "   
						order by totwalltime ASC, sel_legend ASC"; 

		$sql = mysql_query($reqsql) or die(mysql_error());

		while ($row = mysql_fetch_row($sql))
		{
			$data[]=$row;
		}
		return $data;
	}

	/****************************************************************************//**
	 * @brief Selection des groupes
	 *
	 * Requête pour l'affichage des groupes à séléectionner sur la page html
	 * Retourne le champs sel_legend de la table selector
	 *
	 * @param 	aucun
	 * @return	aucun
	 *********************************************************************************/
	public function reqSelector()
	{
		$reqsql = "select sel_legend from selector";
		$this->requeterow($reqsql);

	}

}




?>