<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="fr" lang="fr">
<?php

/****************************************************************************//**
 *
 * @file		index.php
 * @brief 		Fichier principal de l'accounting local
 *
 * Index.php permet de créer des graphes concernant l'utilisation de la grille.
 * Les graphes permettent le suivi de l'occupation des ressources, du nombre de jobs terminés,
 * des durées moyennes des jobs, ...
 *
 * @author		Eric Kieffer <eric.kieffer@ires.in2p3.fr>
 *********************************************************************************/

/****************************************************************************//**
 *
 * Déclation des INCLUDES
 *
 *********************************************************************************/

include	("./inc/accparam.inc.php"); 		// Lecture des paramètres d'environnement
include ("./inc/accexception.inc.php"); 	// Gestion des exceptions
include ("./inc/accgraphparam.inc.php");	// Paramètres des graphes
include ("./inc/accdates.inc.php"); 		// Gestion des dates
include ("./inc/accsql.inc.php");			// Gestion SQL
include ("./inc/accsqliphc.inc.php");		// Gestion SQL
include ("./inc/accsqllapp.inc.php");		// Gestion SQL
include ("./inc/accgraph.inc.php");   		// Gestion des graphes

/****************************************************************************//**
 *
 * Chargement des valeurs par défaut ou initialisation après un Submit
 *
 *********************************************************************************/

try
{
	$sellist="";
	// Déclaration des dates pour le graphique
	$now=time();
	$mdate = new Accdates(date('d-m-Y',$now));
	//$mdate->show();
	$date_yesterday = $mdate->getDateYesterday();
	$mdate2 = new Accdates($date_yesterday);
	$date_monthfirstday  = $mdate2->getFirstDayOfMonth();
	// Valeurs de test pour les dates
	//$date_yesterday = "03-01-2010";
	//$date_monthfirstday  = "01-01-2010";

	// Valeurs après un Submit
	if(isset($_POST['submit']))
	{
		$form_graph_type = $_POST["ListGraphType"];
		$form_date_start = $_POST["datestart"];
		$form_date_end = $_POST["dateend"];
		$form_option_periode = $_POST["optionperiode"];
		$form_option_typejobs = $_POST["optiontypejobs"];
		$form_option_procs = $_POST["optionprocs"];
		$form_option_surcharge = $_POST["optionsurcharge"];
	}
	// Valeurs par défaut à la première connexion
	else
	{
		$form_date_start = $date_monthfirstday;
		$form_date_end = $date_yesterday;
		$form_graph_type = "percwalltime";
		$form_option_typejobs = "all";
		$form_option_periode = "day";
		$form_option_procs = "vprocs";
		$form_option_surcharge = "avec";
	}

	// Selection des spécifités du site
	switch ($param_site)
	{
		case "IPHC":     // Gestion spécifique à l'IPHC
			$msql = new accSqliphc($param_mysql_servername,$param_mysql_dbname,$param_mysql_login,$param_mysql_passwd);
			// Création des vues génériques sql
			$msql->setSgbdAccounting();
			// Création des tables et vues spécifiques à l'IPHC
			$msql->setSgbdIphc();
			break;

		case "LAPP":     // Gestion spécifique au LAPP
			$msql = new accSqllapp($param_mysql_servername,$param_mysql_dbname,$param_mysql_login,$param_mysql_passwd);
			// Création des vues génériques sql
			$msql->setSgbdAccounting();
			break;
	}
	// Connexion à la base de données
	$msql->verifConnexion();

	// Création de la liste de sélection des groupes
	// Tous les groupes sont sélectionnés à la première connexion
	$msql->reqSelector();

	for ($i=0; $i<$msql->rowcount;$i++)
	{

		$temp = $msql->result[$i][0];

		if(isset($_POST['submit']))
		{
			$listgroup=1;
			$trouve=0;
			// Après un Post
			if(isset($_POST["ListGroup"]) && !empty($_POST["ListGroup"]))
			{
				$Col1_Array = $_POST["ListGroup"];
				foreach($Col1_Array as $selectValue)
				{
					if ($temp==$selectValue)
					{
						$trouve=1;
					}
				}
			}
			if ($trouve==1)
			{
				//à modifier
				$sellist .= "<OPTION SELECTED=\"selected\" VALUE='$temp'>$temp</OPTION>";
				$trouve=0;
			}
			else
			{
				$sellist .= "<OPTION VALUE='$temp'>$temp</OPTION>";
				$listgroup=0;
			}
		}
		// Première connexion
		else
		{
			$sellist .= "<OPTION SELECTED=\"selected\" VALUE='$temp'>$temp</OPTION>";
			$listgroup=1;
		}
	}


	/****************************************************************************//**
	 *
	 * HTML
	 * La partie HTML a été déplacé dans le fichier accindexhtml.html pour
	 * plus de lisibilité (à revoir)
	 *
	 *********************************************************************************/

	include ("./inc/accindexhtml.inc.php");

	/****************************************************************************//**
	 *
	 * MAIN
	 * Partie dédiée à la création du graphe
	 *
	 *********************************************************************************/

	// Création du graphe après un Submit
	if(isset($_POST['submit']))
	{
		// Lecture des champs
		$graph_type  = $_POST["ListGraphType"];
		$mdatestart = new Accdates($_POST["datestart"]);
		$mdatestart->convertDate();
		$graph_start = $mdatestart->date;
		$mdateend = new Accdates($_POST["dateend"]);
		$mdateend->convertDate();
		$graph_end   = $mdateend->date;
		$graph_option_periode = $_POST["optionperiode"];
		$graph_jobs =  $_POST["optiontypejobs"];
		$graph_procs = $_POST["optionprocs"];
		$graph_surcharge =  $_POST["optionsurcharge"];
		
			
		// Détermine si des groupes sont sélectionnés
		if ($listgroup==0)
		{
			$graph_group = $_POST["ListGroup"];
		}
		else
		{
			$graph_group = "";
		}
			
		// Création du graphe
		if ($graph_type == "testjpgraph")
		{
			// Gestion du graphe de test (pas de données nécessaire)
			// Permet de tester si l'installation est fonctionnelle
			$accparam=new AccGraphparam($graph_type, "test", "test", "test", "test", "test", "test", "test", $full_graph_filename);
			print $graph_type;
			$data="TEST";
			$mongraph=new AccGraph($accparam, $data);
			$mongraph->createGraph();
		}
		else
		{
			// Gestion des graphes de l'accounting
			// Import des paramètres du graphe
			$accparam=new Accgraphparam($graph_type, $graph_start, $graph_end, $graph_option_periode, $graph_jobs, $graph_surcharge, $graph_group, $graph_procs, $full_graph_filename);
			// Récupération des données Mysql
			$data=$msql->getDataGraph($accparam);

			// Création du graphe
			$mongraph=new Accgraph($accparam, $data);
			$mongraph->createGraph();
		}

		// Affichage du graphe
		$web_graph_filename="graph/graphselection.png";
		//print $web_graph_filename;
		echo "<img src=\"$web_graph_filename\"  alt=\"Selection\" ;/><br><br>";
	}
}

catch (accException $Ae) // Gestion des erreurs
{
	$Ae->getExceptionExit();
}
?>
</body>
</html>
